@@ -32,6 +32,7 @@
 #include <gst/gst.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+#include "rb-player.h"
 #include "rb-metadata.h"
 
 G_BEGIN_DECLS
@@ -48,6 +49,18 @@ gboolean	rb_gst_process_tag_string	(const GstTagList *taglist,
 						 RBMetaDataField *field,
 						 GValue *value);
 
+int		rb_gst_error_get_error_code	(const GError *error);
+
+/* tee and filter support */
+
+GstElement *	rb_gst_create_filter_bin (void);
+
+gboolean	rb_gst_add_filter (RBPlayer *player, GstElement *filterbin, GstElement *element, gboolean use_pad_block);
+gboolean	rb_gst_remove_filter (RBPlayer *player, GstElement *filterbin, GstElement *element, gboolean use_pad_block);
+
+gboolean	rb_gst_add_tee (RBPlayer *player, GstElement *tee, GstElement *element, gboolean use_pad_block);
+gboolean	rb_gst_remove_tee (RBPlayer *player, GstElement *tee, GstElement *element, gboolean use_pad_block);
+
 G_END_DECLS
 
 #endif /* __RB_PLAYER_GST_HELPER_H */
