@@ -438,8 +438,8 @@ extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *cl
                                                                 gpointer      invocation_hint,
                                                                 gpointer      marshal_data);
 
-/* VOID:UINT,STRING,STRING,OBJECT,BOOLEAN (./rb-marshal.list:55) */
-extern void rb_marshal_VOID__UINT_STRING_STRING_OBJECT_BOOLEAN (GClosure     *closure,
+/* VOID:UINT,STRING,STRING,STRING,BOOLEAN (./rb-marshal.list:55) */
+extern void rb_marshal_VOID__UINT_STRING_STRING_STRING_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
                                                                 const GValue *param_values,
@@ -486,6 +486,14 @@ extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closur
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
 
+/* VOID:STRING,UINT,UINT,STRING (./rb-marshal.list:61) */
+extern void rb_marshal_VOID__STRING_UINT_UINT_STRING (GClosure     *closure,
+                                                      GValue       *return_value,
+                                                      guint         n_param_values,
+                                                      const GValue *param_values,
+                                                      gpointer      invocation_hint,
+                                                      gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __rb_marshal_MARSHAL_H__ */
