@@ -60,12 +60,15 @@ typedef enum
 	RB_METADATA_FIELD_ALBUM_GAIN,		   /* double */
 	RB_METADATA_FIELD_ALBUM_PEAK,		   /* double */
 	RB_METADATA_FIELD_LANGUAGE_CODE,	   /* string */
+	RB_METADATA_FIELD_BPM,			   /* double */
 	RB_METADATA_FIELD_MUSICBRAINZ_TRACKID,     /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID,    /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID,     /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID,   /* string */
 	RB_METADATA_FIELD_ARTIST_SORTNAME,         /* string */
-	RB_METADATA_FIELD_ALBUM_SORTNAME,         /* string */
+	RB_METADATA_FIELD_ALBUM_SORTNAME,          /* string */
+	RB_METADATA_FIELD_ALBUM_ARTIST,            /* string */
+	RB_METADATA_FIELD_ALBUM_ARTIST_SORTNAME,   /* string */
 
 	RB_METADATA_FIELD_LAST			   /* nothing */
 } RBMetaDataField;
