@@ -38,7 +38,7 @@ from LastFMCoverArtSearch import LastFMCoverArtSearch
 from EmbeddedCoverArtSearch import EmbeddedCoverArtSearch
 from LocalCoverArtSearch import LocalCoverArtSearch
 
-from urllib import unquote
+from urllib import unquote, pathname2url
 
 ART_SEARCHES_LOCAL = [LocalCoverArtSearch, EmbeddedCoverArtSearch]
 ART_SEARCHES_REMOTE = [PodcastCoverArtSearch, LastFMCoverArtSearch, MusicBrainzCoverArtSearch]
@@ -107,6 +107,13 @@ class TicketSystem:
 			self.dead.remove (ticket)
 			return False
 
+def get_search_props(db, entry):
+	artist = db.entry_get(entry, rhythmdb.PROP_ALBUM_ARTIST)
+	if artist == "":
+		artist = db.entry_get(entry, rhythmdb.PROP_ARTIST)
+	album = db.entry_get(entry, rhythmdb.PROP_ALBUM)
+	return (artist, album)
+
 
 class CoverArtDatabase (object):
 	def __init__ (self):
@@ -114,8 +121,7 @@ class CoverArtDatabase (object):
 		self.same_search = {}
 
 	def build_art_cache_filename (self, db, entry, extension):
-		artist = db.entry_get (entry, rhythmdb.PROP_ARTIST)
-		album = db.entry_get (entry, rhythmdb.PROP_ALBUM)
+		artist, album = get_search_props(db, entry)
 		art_folder = os.path.expanduser (ART_FOLDER)
 		old_art_folder = os.path.expanduser (OLD_ART_FOLDER)
 		if not os.path.exists (art_folder) and os.path.exists (old_art_folder):
@@ -148,6 +154,18 @@ class CoverArtDatabase (object):
 		if entry is None or pixbuf is None:
 			return
 
+		art_location_url = self.cache_pixbuf(db, entry, pixbuf)
+		callback (entry, pixbuf, art_location_url, None, None)
+		for Engine in ART_SEARCHES_LOCAL:
+			try:
+				Engine ().save_pixbuf (db, entry, pixbuf)
+			except AttributeError:
+				pass
+
+	def cache_pixbuf (self, db, entry, pixbuf):
+		if entry is None or pixbuf is None:
+			return None
+
 		meta_location = self.build_art_cache_filename (db, entry, ART_CACHE_EXTENSION_META)
 		self.write_meta_file (meta_location, None, None)
 
@@ -161,12 +179,7 @@ class CoverArtDatabase (object):
 			art_cache_settings = ART_CACHE_SETTINGS_JPG
 		self.ticket.purge (entry)
 		pixbuf.save (art_location, art_cache_format, art_cache_settings)
-		callback (entry, pixbuf, art_location, None, None)
-		for Engine in ART_SEARCHES_LOCAL:
-			try:
-				Engine ().save_pixbuf (db, entry, pixbuf)
-			except AttributeError:
-				pass
+		return "file://" + pathname2url(art_location)
 
 	def cancel_get_pixbuf (self, entry):
 		self.ticket.purge (entry)
@@ -175,9 +188,8 @@ class CoverArtDatabase (object):
 		if entry is None:
 			callback (entry, None, None, None, None)
 			return
-            
-		st_artist = db.entry_get (entry, rhythmdb.PROP_ARTIST) or _("Unknown")
-		st_album = db.entry_get (entry, rhythmdb.PROP_ALBUM) or _("Unknown")
+
+		st_artist, st_album = get_search_props(db, entry)
 
 		# replace quote characters
 		# don't replace single quote: could be important punctuation
@@ -204,17 +216,16 @@ class CoverArtDatabase (object):
 			self.ticket.purge (entry)
 			pixbuf = gtk.gdk.pixbuf_new_from_file (art_location)
 			(tooltip_image, tooltip_text) = self.read_meta_file (art_location_meta)
-			callback (entry, pixbuf, art_location, tooltip_image, tooltip_text)
+			art_location_url = "file://" + pathname2url(art_location)
+			callback (entry, pixbuf, art_location_url, tooltip_image, tooltip_text)
 			return
 
 		# Check if we're already searching for art for this album
 		# (this won't work for compilations, but there isn't much we can do about that)
 		def find_same_search(a, b, db):
-			for prop in (rhythmdb.PROP_ARTIST, rhythmdb.PROP_ALBUM):
-				if db.entry_get(a, prop) != db.entry_get(b, prop):
-					return False
-
-			return True
+			a_artist, a_album = get_search_props(db, a)
+			b_artist, b_album = get_search_props(db, b)
+			return (a_artist == b_artist) and (a_album == b_album)
 
 		match_entry = self.ticket.find(entry, find_same_search, db)
 		if match_entry is not None:
@@ -240,10 +251,10 @@ class CoverArtDatabase (object):
 						if should_save:
 							if pixbuf.get_has_alpha ():
 								pixbuf.save (art_location_png, ART_CACHE_FORMAT_PNG, ART_CACHE_SETTINGS_PNG)
-								uri = art_location_png
+								uri = "file://" + pathname2url(art_location_png)
 							else:
 								pixbuf.save (art_location_jpg, ART_CACHE_FORMAT_JPG, ART_CACHE_SETTINGS_JPG)
-								uri = art_location_jpg
+								uri = "file://" + pathname2url(art_location_jpg)
 
 							self.write_meta_file (art_location_meta, tooltip_image, tooltip_text)
 						else:
