@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header file for Rhythmbox Audioscrobbler support
- *
  *  Copyright (C) 2005 Alex Revo <xiphoiadappendix@gmail.com>
  *					   Ruben Vermeersch <ruben@Lambda1.be>
  *
@@ -9,7 +7,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  * 
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -36,8 +34,8 @@ G_BEGIN_DECLS
 #include <glib.h>
 
 #include "rb-shell-player.h"
-#include "rb-proxy-config.h"
 #include "rb-plugin.h"
+#include "rb-audioscrobbler-service.h"
 
 #define RB_TYPE_AUDIOSCROBBLER			(rb_audioscrobbler_get_type ())
 #define RB_AUDIOSCROBBLER(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_AUDIOSCROBBLER, RBAudioscrobbler))
@@ -59,29 +57,25 @@ typedef struct
 typedef struct
 {
 	GObjectClass parent_class;
+
+	/* signals */
+	void (*authentication_error) (RBAudioscrobbler *audioscrobbler);
+	void (*statistics_changed) (RBAudioscrobbler *audioscrobbler,
+	                            const char *status_msg,
+	                            guint queue_count,
+	                            guint submit_count,
+	                            const char *submit_time);
 } RBAudioscrobblerClass;
 
 
 GType			rb_audioscrobbler_get_type (void);
 
-RBAudioscrobbler *	rb_audioscrobbler_new (RBShellPlayer *shell_player,
-					       RBProxyConfig *proxy_config);
+RBAudioscrobbler *	rb_audioscrobbler_new (RBAudioscrobblerService *service,
+                                               RBShellPlayer *shell_player,
+                                               const char *username,
+                                               const char *session_key);
 
-GtkWidget *		rb_audioscrobbler_get_config_widget (RBAudioscrobbler *audioscrobbler,
-							     RBPlugin *plugin);
-
-void			rb_audioscrobbler_username_entry_changed_cb (GtkEntry *entry,
-								     RBAudioscrobbler *audioscrobbler);
-void			rb_audioscrobbler_username_entry_activate_cb (GtkEntry *entry,
-								      RBAudioscrobbler *audioscrobbler);
-
-void			rb_audioscrobbler_password_entry_changed_cb (GtkEntry *entry,
-								     RBAudioscrobbler *audioscrobbler);
-void			rb_audioscrobbler_password_entry_activate_cb (GtkEntry *entry,
-								      RBAudioscrobbler *audioscrobbler);
-
-void			rb_audioscrobbler_enabled_check_changed_cb (GtkCheckButton *button,
-								    RBAudioscrobbler *audioscrobbler);
+void                    rb_audioscrobbler_statistics_changed (RBAudioscrobbler *audioscrobbler);
 
 
 G_END_DECLS
