@@ -40,8 +40,8 @@
 #include <glib-object.h>
 #include <glib/gstdio.h>
 #include <gdk/gdkx.h>
-#include <brasero/brasero-media.h>
-#include <brasero/brasero-medium-monitor.h>
+#include <brasero-media.h>
+#include <brasero-medium-monitor.h>
 
 #include <libxml/xmlerror.h>
 #include <libxml/xmlwriter.h>
@@ -73,7 +73,7 @@ typedef struct
 	GtkActionGroup *action_group;
 	guint           ui_merge_id;
 
-	RBSource       *selected_source;
+	RBDisplayPage  *selected_page;
 	guint           enabled : 1;
 } RBDiscRecorderPlugin;
 
@@ -159,6 +159,7 @@ rb_disc_recorder_plugin_start_burning (RBDiscRecorderPlugin *pi,
 {
 	GtkWidget *main_window;
 	GdkScreen *screen;
+	GdkWindow *window;
 	GPtrArray *array;
 	char **args, *xid_str;
 	GError *error = NULL;
@@ -172,11 +173,12 @@ rb_disc_recorder_plugin_start_burning (RBDiscRecorderPlugin *pi,
 		g_ptr_array_add (array, "-r");
 	g_ptr_array_add (array, (gpointer) path);
 
-	main_window = gtk_widget_get_toplevel (GTK_WIDGET (pi->selected_source));
+	g_object_get (pi->shell, "window", &main_window, NULL);
 	screen = gtk_widget_get_screen (main_window);
-	if (main_window && main_window->window) {
+	window = gtk_widget_get_window (main_window);
+	if (window) {
 		int xid;
-		xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (GTK_WIDGET (main_window)->window));
+		xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (window));
 		xid_str = g_strdup_printf ("%d", xid);
 		g_ptr_array_add (array, "-x");
 		g_ptr_array_add (array, xid_str);
@@ -188,7 +190,7 @@ rb_disc_recorder_plugin_start_burning (RBDiscRecorderPlugin *pi,
 	args = (char **) g_ptr_array_free (array, FALSE);
 
 	ret = TRUE;
-	if (!gdk_spawn_on_screen (screen, NULL, args, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_FILE_AND_ARGV_ZERO, NULL, NULL, NULL, &error)) {
+	if (!gdk_spawn_on_screen (screen, NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error)) {
 		if (copy != FALSE) {
 			rb_error_dialog (GTK_WINDOW (main_window),
 					 _("Rhythmbox could not duplicate the disc"),
@@ -492,8 +494,8 @@ static void
 cmd_burn_source (GtkAction            *action,
 		 RBDiscRecorderPlugin *pi)
 {
-	if (pi->selected_source != NULL) {
-		source_burn (pi, pi->selected_source);
+	if (pi->selected_page != NULL && RB_IS_SOURCE (pi->selected_page)) {
+		source_burn (pi, RB_SOURCE (pi->selected_page));
 	}
 }
 
@@ -504,10 +506,10 @@ cmd_duplicate_cd (GtkAction         	*action,
 	gchar *device;
 	GVolume *volume;
 
-	if (!pi->selected_source)
+	if (pi->selected_page == NULL || RB_IS_SOURCE (pi->selected_page) == FALSE)
 		return;
 
-	g_object_get (pi->selected_source, "volume", &volume, NULL);
+	g_object_get (pi->selected_page, "volume", &volume, NULL);
 	if (G_IS_VOLUME (volume))
 		device = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	else
@@ -589,25 +591,25 @@ update_source (RBDiscRecorderPlugin *pi,
 {
 	GtkAction *burn_action, *copy_action;
 	gboolean   playlist_active, is_audiocd_active;
-	RBSource  *selected_source;
-	const char *source_type;
+	RBDisplayPage *selected_page;
+	const char *page_type;
 
-	if (pi->selected_source != NULL) {
+	if (pi->selected_page != NULL && RB_IS_SOURCE (pi->selected_page)) {
 		RhythmDBQueryModel *model;
 
-		g_object_get (pi->selected_source, "query-model", &model, NULL);
+		g_object_get (pi->selected_page, "query-model", &model, NULL);
 		g_signal_handlers_disconnect_by_func (model, playlist_row_inserted_cb, pi);
 		g_signal_handlers_disconnect_by_func (model, playlist_entries_changed, pi);
 		g_object_unref (model);
 	}
 
-	g_object_get (shell, "selected-source", &selected_source, NULL);
+	g_object_get (shell, "selected-page", &selected_page, NULL);
 
 	/* for now restrict to playlist sources */
-	playlist_active = RB_IS_PLAYLIST_SOURCE (selected_source);
+	playlist_active = RB_IS_PLAYLIST_SOURCE (selected_page);
 
-	source_type = G_OBJECT_TYPE_NAME (selected_source);
-	is_audiocd_active = g_str_equal (source_type, "RBAudioCdSource");
+	page_type = G_OBJECT_TYPE_NAME (selected_page);
+	is_audiocd_active = g_str_equal (page_type, "RBAudioCdSource");
 
 	burn_action = gtk_action_group_get_action (pi->action_group,
 						   "MusicPlaylistBurnToDiscPlaylist");
@@ -617,7 +619,7 @@ update_source (RBDiscRecorderPlugin *pi,
 	if (pi->enabled && playlist_active && rb_disc_recorder_has_burner (pi)) {
 		RhythmDBQueryModel *model;
 
-		g_object_get (selected_source, "query-model", &model, NULL);
+		g_object_get (selected_page, "query-model", &model, NULL);
 		/* monitor for changes, to enable/disable the burn menu item */
 		g_signal_connect_object (G_OBJECT (model),
 					 "row_inserted",
@@ -641,19 +643,18 @@ update_source (RBDiscRecorderPlugin *pi,
 		gtk_action_set_visible (copy_action, FALSE);
 	}
 
-	if (pi->selected_source != NULL) {
-		g_object_unref (pi->selected_source);
+	if (pi->selected_page != NULL) {
+		g_object_unref (pi->selected_page);
 	}
-	pi->selected_source = selected_source;
+	pi->selected_page = selected_page;
 }
 
 static void
-shell_selected_source_notify_cb (RBShell            *shell,
-				 GParamSpec         *param,
-				 RBDiscRecorderPlugin *pi)
+shell_selected_page_notify_cb (RBShell            *shell,
+			       GParamSpec         *param,
+			       RBDiscRecorderPlugin *pi)
 {
-	rb_debug ("RBDiscRecorderPlugin selected source changed");
-
+	rb_debug ("RBDiscRecorderPlugin selected page changed");
 	update_source (pi, shell);
 }
 
@@ -693,8 +694,8 @@ impl_activate (RBPlugin *plugin,
 		      NULL);
 
 	g_signal_connect_object (G_OBJECT (shell),
-				 "notify::selected-source",
-				 G_CALLBACK (shell_selected_source_notify_cb),
+				 "notify::selected-page",
+				 G_CALLBACK (shell_selected_page_notify_cb),
 				 pi, 0);
 
 	/* add UI */
@@ -753,12 +754,12 @@ impl_deactivate	(RBPlugin *plugin,
 
 	update_source (pi, shell);
 
-	if (pi->selected_source) {
-		g_object_unref (pi->selected_source);
-		pi->selected_source = NULL;
+	if (pi->selected_page) {
+		g_object_unref (pi->selected_page);
+		pi->selected_page = NULL;
 	}
 
-	g_signal_handlers_disconnect_by_func (shell, shell_selected_source_notify_cb, pi);
+	g_signal_handlers_disconnect_by_func (shell, shell_selected_page_notify_cb, pi);
 
 	g_object_get (shell,
 		      "ui-manager", &uimanager,
