@@ -47,7 +47,6 @@ static void     rb_fm_radio_source_class_init  (RBFMRadioSourceClass *class);
 static void     rb_fm_radio_source_init        (RBFMRadioSource *self);
 static void	rb_fm_radio_source_constructed (GObject *object);
 static void     rb_fm_radio_source_dispose     (GObject *object);
-static char    *rb_fm_radio_source_get_playback_uri (RhythmDBEntry *entry, gpointer data);
 static void     rb_fm_radio_source_do_query    (RBFMRadioSource *self);
 static void     rb_fm_radio_source_songs_view_sort_order_changed (
 						RBEntryView *view,
@@ -64,15 +63,15 @@ static void playing_entry_changed (RBShellPlayer *player, RhythmDBEntry *entry,
 				   RBFMRadioSource *self);
 
 static void         impl_delete         (RBSource *source);
-static gboolean     impl_show_popup     (RBSource *source);
-static GList       *impl_get_ui_actions (RBSource *source);
+static gboolean     impl_show_popup     (RBDisplayPage *page);
+static GList       *impl_get_ui_actions (RBDisplayPage *page);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 
 
 struct _RBFMRadioSourcePrivate {
 	RhythmDB *db;
 	RBShellPlayer *player;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDBEntry *playing_entry;
 
 	RBEntryView *stations;
@@ -94,20 +93,22 @@ static void
 rb_fm_radio_source_class_init (RBFMRadioSourceClass *class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
+	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (class);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (class);
 
 	object_class->constructed = rb_fm_radio_source_constructed;
 	object_class->dispose = rb_fm_radio_source_dispose;
 
-	g_type_class_add_private (class, sizeof (RBFMRadioSourcePrivate));
+	page_class->show_popup = impl_show_popup;
+	page_class->get_ui_actions = impl_get_ui_actions;
 
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_entry_view = impl_get_entry_view;
-	source_class->impl_get_ui_actions = impl_get_ui_actions;
+
+	g_type_class_add_private (class, sizeof (RBFMRadioSourcePrivate));
 }
 
 static void
@@ -136,8 +137,8 @@ rb_fm_radio_source_constructed (GObject *object)
 		      NULL);
 	g_object_unref (shell);
 
-	self->priv->action_group = _rb_source_register_action_group (
-		RB_SOURCE (self),
+	self->priv->action_group = _rb_display_page_register_action_group (
+		RB_DISPLAY_PAGE (self),
 		"FMRadioActions",
 		rb_fm_radio_source_actions,
 		G_N_ELEMENTS (rb_fm_radio_source_actions),
@@ -176,23 +177,34 @@ rb_fm_radio_source_constructed (GObject *object)
 				 self, 0);
 }
 
+static char *
+rb_fm_radio_source_get_playback_uri (RhythmDBEntryType *etype, RhythmDBEntry *entry)
+{
+	return g_strdup("xrbsilence:///");
+}
+
 RBSource *
 rb_fm_radio_source_new (RBShell *shell, RBRadioTuner *tuner)
 {
 	RBFMRadioSource *self;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDB *db;
 
 	g_object_get (shell, "db", &db, NULL);
 
 	entry_type = rhythmdb_entry_type_get_by_name (db, "fmradio-station");
-	if (entry_type == RHYTHMDB_ENTRY_TYPE_INVALID) {
-		entry_type = rhythmdb_entry_register_type (db, "fmradio-station");
-		entry_type->save_to_disk = TRUE;
-		entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc) rb_true_function;
-		entry_type->sync_metadata = (RhythmDBEntrySyncFunc) rb_null_function;
+	if (entry_type == NULL) {
+		entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+					   "db", db,
+					   "name", "fmradio-station",
+					   "save-to-disk", TRUE,
+					   NULL);
+		entry_type->can_sync_metadata = (RhythmDBEntryTypeBooleanFunc) rb_true_function;
+		entry_type->sync_metadata = (RhythmDBEntryTypeSyncFunc) rb_null_function;
 		entry_type->get_playback_uri = rb_fm_radio_source_get_playback_uri;
+		rhythmdb_register_entry_type (db, entry_type);
 	}
+
 	self = g_object_new (RB_TYPE_FM_RADIO_SOURCE,
 			     "name", _("FM Radio"),
 			     "shell", shell,
@@ -205,12 +217,6 @@ rb_fm_radio_source_new (RBShell *shell, RBRadioTuner *tuner)
 	return RB_SOURCE (self);
 }
 
-static char *
-rb_fm_radio_source_get_playback_uri (RhythmDBEntry *entry, gpointer data)
-{
-	return g_strdup("xrbsilence:///");
-}
-
 static void
 rb_fm_radio_source_dispose (GObject *object)
 {
@@ -276,9 +282,9 @@ rb_fm_radio_source_songs_view_show_popup (RBEntryView *view,
 		return;
 
 	if (over_entry)
-		_rb_source_show_popup (RB_SOURCE (self), "/FMRadioViewPopup");
+		_rb_display_page_show_popup (RB_DISPLAY_PAGE (self), "/FMRadioViewPopup");
 	else
-		_rb_source_show_popup (RB_SOURCE (self), "/FMRadioSourcePopup");
+		_rb_display_page_show_popup (RB_DISPLAY_PAGE (self), "/FMRadioSourcePopup");
 }
 
 void
@@ -406,14 +412,14 @@ impl_delete (RBSource *source)
 }
 
 static gboolean
-impl_show_popup (RBSource *source)
+impl_show_popup (RBDisplayPage *page)
 {
-	_rb_source_show_popup (source, "/FMRadioSourcePopup");
+	_rb_display_page_show_popup (page, "/FMRadioSourcePopup");
 	return TRUE;
 }
 
 static GList *
-impl_get_ui_actions (RBSource *source)
+impl_get_ui_actions (RBDisplayPage *page)
 {
 	return g_list_prepend (NULL, g_strdup ("MusicNewFMRadioStation"));
 }
