@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of PSP source object
  *
  *  Copyright (C) 2006 James Livingston  <doclivingston@gmail.com>
  *
@@ -78,10 +76,10 @@ rb_psp_source_init (RBPspSource *source)
 }
 
 RBRemovableMediaSource *
-rb_psp_source_new (RBShell *shell, GMount *mount, MPIDDevice *device_info)
+rb_psp_source_new (RBPlugin *plugin, RBShell *shell, GMount *mount, MPIDDevice *device_info)
 {
 	RBPspSource *source;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDB *db;
 	char *name;
 	char *path;
@@ -94,19 +92,23 @@ rb_psp_source_new (RBShell *shell, GMount *mount, MPIDDevice *device_info)
 	g_object_get (G_OBJECT (shell), "db", &db, NULL);
 	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	name = g_strdup_printf ("psp: %s", path);
-	entry_type = rhythmdb_entry_register_type (db, name);
+	entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+				   "db", db,
+				   "name", name,
+				   "save-to-disk", FALSE,
+				   "category", RHYTHMDB_ENTRY_NORMAL,
+				   NULL);
+	rhythmdb_register_entry_type (db, entry_type);
 	g_object_unref (db);
 	g_free (name);
 	g_free (path);
 	g_object_unref (volume);
 
 	source = RB_PSP_SOURCE (g_object_new (RB_TYPE_PSP_SOURCE,
+					  "plugin", plugin,
 					  "entry-type", entry_type,
-					  "ignore-entry-type", RHYTHMDB_ENTRY_TYPE_INVALID,
-					  "error-entry-type", RHYTHMDB_ENTRY_TYPE_INVALID,
 					  "mount", mount,
 					  "shell", shell,
-					  "source-group", RB_SOURCE_GROUP_DEVICES,
 					  "device-info", device_info,
 					  NULL));
 
@@ -196,7 +198,7 @@ visit_playlist_dirs (RBPspSource *source,
 {
 	RBShell *shell;
 	RhythmDB *db;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	char *playlist_path;
 	char *playlist_name;
 	RBSource *playlist;
@@ -219,7 +221,7 @@ visit_playlist_dirs (RBPspSource *source,
 				      RHYTHMDB_QUERY_PROP_PREFIX, RHYTHMDB_PROP_LOCATION, playlist_path,
 				      RHYTHMDB_QUERY_END);
 	g_free (playlist_path);
-        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+        g_object_unref (entry_type);
 
 	playlist_name = g_file_get_basename (file);
 	playlist = rb_auto_playlist_source_new (shell, playlist_name, FALSE);
@@ -276,7 +278,7 @@ gboolean
 rb_psp_is_mount_player (GMount *mount, MPIDDevice *device_info)
 {
 	char *model;
-	gboolean result;
+	gboolean result = FALSE;
 
 	g_object_get (device_info, "model", &model, NULL);
 	if (model != NULL && (g_str_equal (model, "PSP") || g_str_equal (model, "\"PSP\" MS"))) {
@@ -284,6 +286,6 @@ rb_psp_is_mount_player (GMount *mount, MPIDDevice *device_info)
 	}
 	g_free (model);
 
-	return FALSE;
+	return result;
 }
 
