@@ -53,6 +53,8 @@
 #include "rb-playlist-source.h"
 #include "rb-playlist-manager.h"
 #include "rb-podcast-manager.h"
+#include "rb-podcast-entry-types.h"
+#include "rb-stock-icons.h"
 
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/ipod/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/ipod/show_browser"
@@ -64,11 +66,12 @@ static void rb_ipod_source_dispose (GObject *object);
 static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 
-static gboolean impl_show_popup (RBSource *source);
 static void impl_delete (RBSource *asource);
 static void rb_ipod_load_songs (RBiPodSource *source);
-static void impl_delete_thyself (RBSource *source);
-static GList* impl_get_ui_actions (RBSource *source);
+
+static gboolean impl_show_popup (RBDisplayPage *page);
+static void impl_delete_thyself (RBDisplayPage *page);
+static GList* impl_get_ui_actions (RBDisplayPage *page);
 
 static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
@@ -159,6 +162,7 @@ static void
 rb_ipod_source_class_init (RBiPodSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBMediaPlayerSourceClass *mps_class = RB_MEDIA_PLAYER_SOURCE_CLASS (klass);
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
@@ -170,13 +174,14 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	object_class->set_property = rb_ipod_source_set_property;
 	object_class->get_property = rb_ipod_source_get_property;
 
+	page_class->delete_thyself = impl_delete_thyself;
+	page_class->show_popup = impl_show_popup;
+	page_class->get_ui_actions = impl_get_ui_actions;
+
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
-	source_class->impl_show_popup = impl_show_popup;
-	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_get_ui_actions = impl_get_ui_actions;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
 
@@ -278,7 +283,7 @@ rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 
 static void
 rb_ipod_source_init (RBiPodSource *source)
-{	
+{
 }
 
 static void
@@ -358,7 +363,7 @@ rb_ipod_source_new (RBPlugin *plugin,
 		    MPIDDevice *device_info)
 {
 	RBiPodSource *source;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDB *db;
 	GVolume *volume;
 	char *name;
@@ -369,12 +374,16 @@ rb_ipod_source_new (RBPlugin *plugin,
 	if (path == NULL)
 		path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UUID);
 	g_object_unref (volume);
-	
+
 	g_object_get (shell, "db", &db, NULL);
 	name = g_strdup_printf ("ipod: %s", path);
-	entry_type =  rhythmdb_entry_register_type (db, name);
-	entry_type->save_to_disk = FALSE;
-	entry_type->category = RHYTHMDB_ENTRY_NORMAL;
+	entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+				   "db", db,
+				   "name", name,
+				   "save-to-disk", FALSE,
+				   "category", RHYTHMDB_ENTRY_NORMAL,
+				   NULL);
+	rhythmdb_register_entry_type (db, entry_type);
 	g_object_unref (db);
 	g_free (name);
 	g_free (path);
@@ -384,12 +393,11 @@ rb_ipod_source_new (RBPlugin *plugin,
 					       "entry-type", entry_type,
 					       "mount", mount,
 					       "shell", shell,
-					       "source-group", RB_SOURCE_GROUP_DEVICES,
 					       "device-info", device_info,
 					       NULL));
 
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
-        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+        g_object_unref (entry_type);
 
 	return RB_MEDIA_PLAYER_SOURCE (source);
 }
@@ -465,6 +473,150 @@ playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
 	rb_ipod_db_add_to_playlist (priv->ipod_db, ipod_pl, track);
 }
 
+static void
+playlist_before_save (RbIpodDb *ipod_db, gpointer data)
+{
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	GtkTreeModel *model = GTK_TREE_MODEL (rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist)));
+	Itdb_Playlist *ipod_pl = rb_ipod_static_playlist_source_get_itdb_playlist (playlist);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+	GtkTreeIter iter;
+
+	if (!rb_ipod_static_playlist_source_get_was_reordered (playlist)) {
+		return;
+	}
+
+	rb_ipod_static_playlist_source_set_was_reordered (playlist, FALSE);
+
+	/* Sanity check that all tracks are in entry_map */
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			RhythmDBEntry *entry;
+			Itdb_Track *track;
+
+			gtk_tree_model_get (model, &iter, 0, &entry, -1);
+			track = g_hash_table_lookup (priv->entry_map, entry);
+
+			g_return_if_fail (track != NULL);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+
+	/* Remove all tracks then re-add in correct order */
+
+	while (ipod_pl->members != NULL) {
+		Itdb_Track *track;
+
+		track = (Itdb_Track *)ipod_pl->members->data;
+
+		rb_debug ("removing \"%s\" from \"%s\"", track->title, ipod_pl->name);
+
+		/* Call directly to itdb to avoid scheduling another save */
+		itdb_playlist_remove_track (ipod_pl, track);
+	}
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			RhythmDBEntry *entry;
+			Itdb_Track *track;
+
+			gtk_tree_model_get (model, &iter, 0, &entry, -1);
+			track = g_hash_table_lookup (priv->entry_map, entry);
+
+			rb_debug ("adding \"%s\" to \"%s\"", track->title, ipod_pl->name);
+
+			/* Call directly to itdb to avoid scheduling another save */
+			itdb_playlist_add_track (ipod_pl, track, -1);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+}
+
+static void
+playlist_rows_reordered (GtkTreeModel *model,
+			 GtkTreePath *path,
+			 GtkTreeIter *iter,
+			 gint *order, gpointer data)
+{
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+
+	/* Mark as reordered; will check before save */
+	rb_ipod_static_playlist_source_set_was_reordered (playlist, TRUE);
+
+	rb_ipod_db_save_async (priv->ipod_db);
+}
+
+static void
+playlist_source_model_disconnect_signals (GObject *model, RBIpodStaticPlaylistSource *playlist_source)
+{
+	g_return_if_fail (RB_IS_IPOD_STATIC_PLAYLIST_SOURCE (playlist_source));
+
+	g_signal_handlers_disconnect_by_func (model,
+					      G_CALLBACK (playlist_track_added),
+					      playlist_source);
+	g_signal_handlers_disconnect_by_func (model,
+					      G_CALLBACK (playlist_track_removed),
+					      playlist_source);
+	g_signal_handlers_disconnect_by_func (model,
+					      G_CALLBACK (playlist_rows_reordered),
+					      playlist_source);
+}
+
+static void
+playlist_source_model_connect_signals (RBIpodStaticPlaylistSource *playlist_source)
+{
+	RhythmDBQueryModel *model;
+
+	g_return_if_fail (RB_IS_IPOD_STATIC_PLAYLIST_SOURCE (playlist_source));
+
+	g_object_get (G_OBJECT (playlist_source),
+		      "base-query-model", &model, NULL);
+	g_signal_connect (model, "row-inserted",
+			  G_CALLBACK (playlist_track_added),
+			  playlist_source);
+	g_signal_connect (model, "entry-removed",
+			  G_CALLBACK (playlist_track_removed),
+			  playlist_source);
+	g_signal_connect (model, "rows-reordered",
+			  G_CALLBACK (playlist_rows_reordered),
+			  playlist_source);
+	g_object_unref (model);
+}
+
+static void
+playlist_source_model_changed (GObject *obj, GParamSpec *pspec, gpointer old_model)
+{
+	RBIpodStaticPlaylistSource *playlist_source;
+
+	rb_debug ("base model changed for iPod playlist");
+
+	playlist_source = RB_IPOD_STATIC_PLAYLIST_SOURCE (obj);
+
+	playlist_source_model_disconnect_signals (G_OBJECT (old_model), playlist_source);
+
+	playlist_source_model_connect_signals (playlist_source);
+}
+
+static void
+set_podcast_icon (RBIpodStaticPlaylistSource *source)
+{
+	GdkPixbuf *pixbuf;
+	gint       size;
+
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
+	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+					   RB_STOCK_PODCAST,
+					   size,
+					   0, NULL);
+
+	if (pixbuf != NULL) {
+		g_object_set (source, "pixbuf", pixbuf, NULL);
+		g_object_unref (pixbuf);
+	}
+}
+
 static RBIpodStaticPlaylistSource *
 add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 {
@@ -472,7 +624,7 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	RBIpodStaticPlaylistSource *playlist_source;
 	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDBQueryModel *model;
 
 	g_object_get (source,
@@ -485,7 +637,7 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
                                                               priv->ipod_db,
                                                               playlist,
                                                               entry_type);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	for (it = playlist->members; it != NULL; it = it->next) {
 		Itdb_Track *song;
@@ -509,17 +661,22 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	playlist->userdata_destroy = g_object_unref;
 	playlist->userdata_duplicate = g_object_ref;
 
-	model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist_source));
-	g_signal_connect (model, "row-inserted",
-			          G_CALLBACK (playlist_track_added),
-			          playlist_source);
-	g_signal_connect (model, "entry-removed",
-			          G_CALLBACK (playlist_track_removed),
-			          playlist_source);
+	g_object_get (G_OBJECT (playlist_source),
+		      "base-query-model", &model, NULL);
+	g_signal_connect (playlist_source, "notify::base-query-model",
+			  G_CALLBACK (playlist_source_model_changed),
+			  playlist_source);
+	g_signal_connect (priv->ipod_db, "before-save",
+			  G_CALLBACK (playlist_before_save),
+			  playlist_source);
+	g_object_unref (model);
+	playlist_source_model_connect_signals (playlist_source);
 
-	if (itdb_playlist_is_podcasts(playlist))
+	if (itdb_playlist_is_podcasts(playlist)) {
 		priv->podcast_pl = playlist_source;
-	rb_shell_append_source (shell, RB_SOURCE (playlist_source), RB_SOURCE (source));
+		set_podcast_icon (playlist_source);
+	}
+	rb_shell_append_display_page (shell, RB_DISPLAY_PAGE (playlist_source), RB_DISPLAY_PAGE (source));
 	g_object_unref (shell);
 
 	return playlist_source;
@@ -548,10 +705,6 @@ load_ipod_playlists (RBiPodSource *source)
 
 }
 
-/* FIXME:  these should go away once we compile against new-enough libgpod */
-#define MEDIATYPE_AUDIO         0x0001
-#define MEDIATYPE_PODCAST       0x0004
-
 static Itdb_Track *
 create_ipod_song_from_entry (RhythmDBEntry *entry, guint64 filesize, const char *mimetype)
 {
@@ -585,11 +738,11 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, guint64 filesize, const char
 	track->app_rating = track->rating;
 	track->playcount = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_PLAY_COUNT);
 
-	if (rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
-		track->mediatype = MEDIATYPE_PODCAST;
+	if (rhythmdb_entry_get_entry_type (entry) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
+		track->mediatype = ITDB_MEDIATYPE_PODCAST;
 		track->time_released = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
 	} else {
-		track->mediatype = MEDIATYPE_AUDIO;
+		track->mediatype = ITDB_MEDIATYPE_AUDIO;
 	}
 
 	return track;
@@ -621,19 +774,17 @@ static void
 add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 {
 	RhythmDBEntry *entry;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	char *pc_path;
 	const char *mount_path;
 
 	/* Set URI */
-	g_object_get (source, "entry-type", &entry_type,
-		      NULL);
+	g_object_get (source, "entry-type", &entry_type, NULL);
 	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
 	pc_path = ipod_path_to_uri (mount_path, song->ipod_path);
-	entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type,
-				    pc_path);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type, pc_path);
+	g_object_unref (entry_type);
 
 	if (entry == NULL) {
 		rb_debug ("cannot create entry %s", pc_path);
@@ -641,6 +792,12 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		return;
 	}
 
+	if ((song->mediatype != ITDB_MEDIATYPE_AUDIO)
+	    && (song->mediatype != ITDB_MEDIATYPE_PODCAST)) {
+		rb_debug ("iPod track is neither an audio track nor a podcast, skipping");
+		return;
+	}
+
 	rb_debug ("Adding %s from iPod", pc_path);
 	g_free (pc_path);
 
@@ -853,7 +1010,7 @@ remove_playcount_file (RBiPodSource *source)
         char *playcounts_file;
         int result;
 	const char *mountpoint;
-	
+
 	mountpoint = rb_ipod_db_get_mount_path (priv->ipod_db);
         itunesdb_dir = itdb_get_itunes_dir (mountpoint);
         playcounts_file = itdb_get_path (itunesdb_dir, "Play Counts");
@@ -918,24 +1075,20 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 	int i;
 
 	/* Ignore entries which are not iPod entries */
-	RhythmDBEntryType entry_type;
-	RhythmDBEntryType ipod_entry_type;
+	RhythmDBEntryType *entry_type;
+	RhythmDBEntryType *ipod_entry_type;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
-	entry_type = rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE);
-	g_object_get (G_OBJECT (source),
-		      "entry-type", &ipod_entry_type,
-		      NULL);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, ipod_entry_type);
-	/* ipod_entry_type can no longer be dereferenced, but its value 
-	 * hasn't been changed, so this comparison is valid
-	 */
+	entry_type = rhythmdb_entry_get_entry_type (entry);
+	g_object_get (source, "entry-type", &ipod_entry_type, NULL);
 	if (entry_type != ipod_entry_type) {
-		return; 
+		g_object_unref (ipod_entry_type);
+		return;
 	}
+	g_object_unref (ipod_entry_type);
 
 	/* If an interesting property was changed, update it on the iPod */
-	/* If the iPod database is being saved in a separate thread, this 
+	/* If the iPod database is being saved in a separate thread, this
 	 * might not be 100% thread-safe, but at worst we'll modify a field
 	 * at the time it's being saved which will get a wrong value, but
 	 * that's the worst that can happen and that's pretty theoretical,
@@ -953,7 +1106,7 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 			old_rating = g_value_get_double (&change->old);
 			new_rating = g_value_get_double (&change->new);
 			if (old_rating != new_rating) {
-				track = g_hash_table_lookup (priv->entry_map, 
+				track = g_hash_table_lookup (priv->entry_map,
 							     entry);
 				track->rating = new_rating * ITDB_RATING_STEP;
 				track->app_rating = track->rating;
@@ -972,7 +1125,7 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 			old_playcount = g_value_get_ulong (&change->old);
 			new_playcount = g_value_get_ulong (&change->new);
 			if (old_playcount != new_playcount) {
-				track = g_hash_table_lookup (priv->entry_map, 
+				track = g_hash_table_lookup (priv->entry_map,
 							     entry);
 				track->playcount = new_playcount;
 				rb_debug ("playcount changed, saving db");
@@ -990,7 +1143,7 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 			old_lastplay = g_value_get_ulong (&change->old);
 			new_lastplay = g_value_get_ulong (&change->new);
 			if (old_lastplay != new_lastplay) {
-				track = g_hash_table_lookup (priv->entry_map, 
+				track = g_hash_table_lookup (priv->entry_map,
 							     entry);
 				track->time_played = new_lastplay;
 				rb_debug ("last play time changed, saving db");
@@ -998,7 +1151,7 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 			} else {
 				rb_debug ("last play time didn't change");
 			}
-			break;			
+			break;
 		}
 		default:
 			rb_debug ("Ignoring property %d", change->prop);
@@ -1028,7 +1181,7 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 	load_ipod_playlists (source);
 	send_offline_plays_notification (source);
 
-	g_signal_connect_object(G_OBJECT(db), "entry-changed", 
+	g_signal_connect_object(G_OBJECT(db), "entry-changed",
 				G_CALLBACK (rb_ipod_source_entry_changed_cb),
 				source, 0);
 
@@ -1066,7 +1219,7 @@ rb_ipod_load_songs (RBiPodSource *source)
 }
 
 static GList*
-impl_get_ui_actions (RBSource *source)
+impl_get_ui_actions (RBDisplayPage *page)
 {
 	GList *actions = NULL;
 
@@ -1089,9 +1242,9 @@ impl_get_paned_key (RBBrowserSource *source)
 }
 
 static gboolean
-impl_show_popup (RBSource *source)
+impl_show_popup (RBDisplayPage *page)
 {
-	_rb_source_show_popup (RB_SOURCE (source), "/iPodSourcePopup");
+	_rb_display_page_show_popup (page, "/iPodSourcePopup");
 	return TRUE;
 }
 
@@ -1223,8 +1376,7 @@ impl_remove_playlists (RBMediaPlayerSource *source)
 		    !playlist->is_spl) {
 
 			/* destroy the playlist source */
-			RBSource *rb_playlist = RB_SOURCE (playlist->userdata);
-			rb_source_delete_thyself (rb_playlist);
+			rb_display_page_delete_thyself (RB_DISPLAY_PAGE (playlist->userdata));
 
 			/* remove playlist from ipod */
 			rb_ipod_db_remove_playlist (priv->ipod_db, playlist);
@@ -1251,7 +1403,7 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 
 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
-	dest = ipod_get_filename_for_uri (mount_path,  uri, 
+	dest = ipod_get_filename_for_uri (mount_path,  uri,
 					  mimetype, extension);
 	if (dest != NULL) {
 		char *dest_uri;
@@ -1277,7 +1429,7 @@ artwork_notify_cb (RhythmDB *db,
 
 	g_return_if_fail (G_VALUE_HOLDS (metadata, GDK_TYPE_PIXBUF));
 	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
-		
+
 	song = g_hash_table_lookup (priv->artwork_request_map, entry);
 	if (song == NULL)
 		return;
@@ -1293,29 +1445,29 @@ rb_add_artwork_whole_album_cb (GtkTreeModel *query_model,
 			       RBiPodSongArtworkAddData *artwork_data)
 {
 	RhythmDBEntry *entry;
-	Itdb_Track *song;	
-	
+	Itdb_Track *song;
+
 	entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (query_model), iter);
 
 	song = g_hash_table_lookup (artwork_data->priv->entry_map, entry);
 	rhythmdb_entry_unref (entry);
 	g_return_val_if_fail (song != NULL, FALSE);
 
-	if (song->has_artwork == 0x01) {		   
-		return FALSE;				
+	if (song->has_artwork == 0x01) {
+		return FALSE;
 	}
-	
-	rb_ipod_db_set_thumbnail (artwork_data->priv->ipod_db, song, artwork_data->pixbuf);
-	
-	return FALSE;	
-}			     
 
-static gboolean 
+	rb_ipod_db_set_thumbnail (artwork_data->priv->ipod_db, song, artwork_data->pixbuf);
+
+	return FALSE;
+}
+
+static gboolean
 rb_ipod_song_artwork_add_cb (RhythmDB *db,
 			     RhythmDBEntry *entry,
 			     const gchar *property_name,
 			     const GValue *metadata,
-			     RBiPodSource *isource)			     
+			     RBiPodSource *isource)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
 	Itdb_Device *device;
@@ -1323,7 +1475,7 @@ rb_ipod_song_artwork_add_cb (RhythmDB *db,
 	GdkPixbuf *pixbuf;
 	GtkTreeModel *query_model;
 	RBiPodSongArtworkAddData artwork_data;
-        RhythmDBEntryType entry_type;
+        RhythmDBEntryType *entry_type;
 
 	if (metadata == NULL) {
 		return FALSE;
@@ -1333,7 +1485,7 @@ rb_ipod_song_artwork_add_cb (RhythmDB *db,
 		return FALSE;
 	}
 
-	song = g_hash_table_lookup (priv->entry_map, entry);		
+	song = g_hash_table_lookup (priv->entry_map, entry);
         if (song == NULL) {
                 return FALSE;
         }
@@ -1347,12 +1499,12 @@ rb_ipod_song_artwork_add_cb (RhythmDB *db,
 		return FALSE;
 	}
 
-        g_object_get (G_OBJECT (isource), "entry-type", &entry_type, NULL);
- 
+        g_object_get (isource, "entry-type", &entry_type, NULL);
+
 	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
-	
+
 	query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty (db));
-	
+
 	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (query_model),
                                 RHYTHMDB_QUERY_PROP_EQUALS,
                                 RHYTHMDB_PROP_TYPE, entry_type,
@@ -1361,18 +1513,18 @@ rb_ipod_song_artwork_add_cb (RhythmDB *db,
 				RHYTHMDB_QUERY_PROP_EQUALS,
 				RHYTHMDB_PROP_ALBUM, song->album,
 				RHYTHMDB_QUERY_END);
-				
+
 	artwork_data.priv = priv;
 	artwork_data.pixbuf = pixbuf;
-				
+
 	gtk_tree_model_foreach (query_model,
 				(GtkTreeModelForeachFunc) rb_add_artwork_whole_album_cb,
 				&artwork_data);
-        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);	
+        g_object_unref (entry_type);
 	g_object_unref(query_model);
 	return FALSE;
 }
-												
+
 static void
 request_artwork (RBiPodSource *isource,
 		 RhythmDBEntry *entry,
@@ -1472,10 +1624,10 @@ impl_track_added (RBRemovableMediaSource *source,
 							    filename);
 		g_free (filename);
 
-		if (song->mediatype == MEDIATYPE_PODCAST) {
+		if (song->mediatype == ITDB_MEDIATYPE_PODCAST) {
 			add_to_podcasts (isource, song);
 		}
-		device = rb_ipod_db_get_device (priv->ipod_db);		
+		device = rb_ipod_db_get_device (priv->ipod_db);
 		if (device && itdb_device_supports_artwork (device)) {
 			request_artwork (isource, entry, db, song);
 		}
@@ -1485,9 +1637,6 @@ impl_track_added (RBRemovableMediaSource *source,
 
 	g_object_unref (db);
 
-	/* chain up to parent class for sync */
-	RB_REMOVABLE_MEDIA_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_track_added (source, entry, dest, filesize, mimetype);
-
 	return FALSE;
 }
 
@@ -1750,21 +1899,21 @@ ipod_path_from_unix_path (const gchar *mount_point, const gchar *unix_path)
 }
 
 static void
-impl_delete_thyself (RBSource *source)
+impl_delete_thyself (RBDisplayPage *page)
 {
-	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (page);
 	RhythmDB *db;
 	GList *p;
 
-        if (priv->ipod_db == NULL) {
-            RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
-            return;
-        }
+	if (priv->ipod_db == NULL) {
+		RB_DISPLAY_PAGE_CLASS (rb_ipod_source_parent_class)->delete_thyself (page);
+		return;
+	}
 
-	db = get_db_for_source (RB_IPOD_SOURCE (source));
+	db = get_db_for_source (RB_IPOD_SOURCE (page));
 	g_signal_handlers_disconnect_by_func (db,
 					      G_CALLBACK (rb_ipod_song_artwork_add_cb),
-					      RB_IPOD_SOURCE (source));
+					      RB_IPOD_SOURCE (page));
 	g_object_unref (db);
 
 	for (p = rb_ipod_db_get_playlists (priv->ipod_db);
@@ -1776,23 +1925,22 @@ impl_delete_thyself (RBSource *source)
 			RhythmDBQueryModel *model;
 
 			rb_playlist = RB_SOURCE (playlist->userdata);
-			model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (rb_playlist));
+			g_object_get (G_OBJECT (rb_playlist),
+				      "base-query-model", &model, NULL);
 
 			/* remove these to ensure they aren't called during source deletion */
-			g_signal_handlers_disconnect_by_func (model,
-							      G_CALLBACK (playlist_track_added),
-							      rb_playlist);
-			g_signal_handlers_disconnect_by_func (model,
-							      G_CALLBACK (playlist_track_removed),
-							      rb_playlist);
-			rb_source_delete_thyself (rb_playlist);
+			playlist_source_model_disconnect_signals (G_OBJECT (model),
+								  RB_IPOD_STATIC_PLAYLIST_SOURCE (rb_playlist));
+
+			g_object_unref (model);
+			rb_display_page_delete_thyself (RB_DISPLAY_PAGE (rb_playlist));
 		}
 	}
 
 	g_object_unref (G_OBJECT (priv->ipod_db));
 	priv->ipod_db = NULL;
 
-	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
+	RB_DISPLAY_PAGE_CLASS (rb_ipod_source_parent_class)->delete_thyself (page);
 }
 
 static GList *
@@ -1831,16 +1979,16 @@ rb_ipod_source_remove_playlist (RBiPodSource *ipod_source,
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod_source);
 	RBIpodStaticPlaylistSource *playlist_source = RB_IPOD_STATIC_PLAYLIST_SOURCE (source);
 
-	rb_source_delete_thyself (source);
+	rb_display_page_delete_thyself (RB_DISPLAY_PAGE (source));
 	rb_ipod_db_remove_playlist (priv->ipod_db, rb_ipod_static_playlist_source_get_itdb_playlist (playlist_source));
 }
 
 static gboolean
 ipod_name_changed_cb (GtkWidget     *widget,
  		      GdkEventFocus *event,
- 		      gpointer       user_data) 
+		      gpointer       user_data)
 {
-	g_object_set (RB_SOURCE (user_data), "name", 
+	g_object_set (RB_SOURCE (user_data), "name",
 		      gtk_entry_get_text (GTK_ENTRY (widget)),
 		      NULL);
 	return FALSE;
@@ -1888,7 +2036,7 @@ impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidge
  		rb_debug ("Couldn't load ipod-info.ui");
  		return;
  	}
-	
+
 	ipod_dev = rb_ipod_db_get_device (priv->ipod_db);
 
 	/* basic tab stuff */
@@ -1904,7 +2052,7 @@ impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidge
 	g_hash_table_iter_init (&iter, priv->entry_map);
 	while (g_hash_table_iter_next (&iter, &key, &value)) {
 		Itdb_Track *track = value;
-		if (track->mediatype == MEDIATYPE_PODCAST) {
+		if (track->mediatype == ITDB_MEDIATYPE_PODCAST) {
 			num_podcasts++;
 		}
 	}
@@ -1982,13 +2130,23 @@ get_mount_point	(RBiPodSource *source)
 static guint64
 impl_get_capacity (RBMediaPlayerSource *source)
 {
-	return rb_ipod_helpers_get_capacity (get_mount_point (RB_IPOD_SOURCE (source)));
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	if (priv->ipod_db) {
+		return rb_ipod_helpers_get_capacity (get_mount_point (RB_IPOD_SOURCE (source)));
+	} else {
+		return 0;
+	}
 }
 
 static guint64
 impl_get_free_space (RBMediaPlayerSource *source)
 {
-	return rb_ipod_helpers_get_free_space (get_mount_point (RB_IPOD_SOURCE (source)));
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	if (priv->ipod_db) {
+		return rb_ipod_helpers_get_free_space (get_mount_point (RB_IPOD_SOURCE (source)));
+	} else {
+		return 0;
+	}
 }
 
 static void
