@@ -59,7 +59,7 @@
 #define TRAY_ICON_DEFAULT_TOOLTIP _("Music Player")
 
 #define TOOLTIP_IMAGE_BORDER_WIDTH	1
-#define PLAYING_ENTRY_NOTIFY_TIME 	4
+#define PLAYING_ENTRY_NOTIFY_TIME	4
 
 #define CONF_PLUGIN_SETTINGS	"/apps/rhythmbox/plugins/status-icon"
 #define CONF_NOTIFICATION_MODE	CONF_PLUGIN_SETTINGS "/notification-mode"
@@ -75,6 +75,7 @@ static void toggle_window_cmd (GtkAction *action, RBStatusIconPlugin *plugin);
 static void show_window_cmd (GtkAction *action, RBStatusIconPlugin *plugin);
 static void show_notifications_cmd (GtkAction *action, RBStatusIconPlugin *plugin);
 static void update_status_icon_visibility (RBStatusIconPlugin *plugin, gboolean notifying);
+static void notify_playing_entry (RBStatusIconPlugin *plugin, gboolean requested);
 
 struct _RBStatusIconPluginPrivate
 {
@@ -94,7 +95,7 @@ struct _RBStatusIconPluginPrivate
 		ICON_WITH_NOTIFY,
 		ICON_ALWAYS,
 		ICON_OWNS_WINDOW
-	} icon_mode;
+	} icon_mode, conf_icon_mode;
 	enum {
 		NOTIFY_NEVER = 0,
 		NOTIFY_HIDDEN,
@@ -116,11 +117,14 @@ struct _RBStatusIconPluginPrivate
 	gboolean tooltips_suppressed;
 
 	/* notification data */
-	GdkPixbuf *notify_pixbuf;
+	gchar *notify_art_path;
 #ifdef HAVE_NOTIFY
 	NotifyNotification *notification;
 	gboolean notify_supports_actions;
+	gboolean notify_supports_icon_buttons;
+	gboolean notify_supports_persistence;
 #endif
+	gboolean is_gnome_shell;
 
 	GtkWidget *config_dialog;
 	GtkWidget *notify_combo;
@@ -326,6 +330,10 @@ visibility_changed_cb (RBShell *shell,
 		       RBStatusIconPlugin *plugin)
 {
 	sync_actions (plugin);
+
+	if (visible == FALSE && plugin->priv->notify_supports_persistence) {
+		notify_playing_entry (plugin, FALSE);
+	}
 }
 
 /* notification popups */
@@ -340,6 +348,10 @@ notification_closed_cb (NotifyNotification *notification,
 	rb_tray_icon_trigger_tooltip_query (plugin->priv->tray_icon);
 
 	update_status_icon_visibility (plugin, FALSE);
+
+	if (notification != plugin->priv->notification) {
+		g_object_unref (notification);
+	}
 }
 
 static void
@@ -351,34 +363,75 @@ notification_next_cb (NotifyNotification *notification,
 	rb_shell_player_do_next (plugin->priv->shell_player, NULL);
 }
 
+static void
+notification_playpause_cb (NotifyNotification *notification,
+			   const char *action,
+			   RBStatusIconPlugin *plugin)
+{
+	rb_debug ("notification action: %s", action);
+	rb_shell_player_playpause (plugin->priv->shell_player, FALSE, NULL);
+}
+
+static void
+notification_previous_cb (NotifyNotification *notification,
+			  const char *action,
+			  RBStatusIconPlugin *plugin)
+{
+	rb_debug ("notification action: %s", action);
+	rb_shell_player_do_previous (plugin->priv->shell_player, NULL);
+}
+
+static gboolean
+init_notify (RBStatusIconPlugin *plugin)
+{
+	if (notify_is_initted ()) {
+		return TRUE;
+	}
+	GList *caps;
+
+	if (notify_init ("Rhythmbox") == FALSE) {
+		g_warning ("libnotify initialization failed");
+		return FALSE;
+	}
+
+	/* ask the notification server if it supports actions */
+	caps = notify_get_server_caps ();
+	if (g_list_find_custom (caps, "actions", (GCompareFunc)g_strcmp0) != NULL) {
+		rb_debug ("notification server supports actions");
+		plugin->priv->notify_supports_actions = TRUE;
+
+		if ((g_list_find_custom (caps, "x-gnome-icon-buttons", (GCompareFunc)g_strcmp0) != NULL) ||
+		    (g_list_find_custom (caps, "action-icons", (GCompareFunc)g_strcmp0) != NULL)) {
+			rb_debug ("notifiction server supports icon buttons");
+			plugin->priv->notify_supports_icon_buttons = TRUE;
+		}
+	} else {
+		rb_debug ("notification server does not support actions");
+	}
+	if (g_list_find_custom (caps, "persistence", (GCompareFunc)g_strcmp0) != NULL) {
+		rb_debug ("notification server supports persistence");
+		plugin->priv->notify_supports_persistence = TRUE;
+	} else {
+		rb_debug ("notification server does not support persistence");
+	}
+
+	rb_list_deep_free (caps);
+	return TRUE;
+}
+
 static void
 do_notify (RBStatusIconPlugin *plugin,
 	   guint timeout,
 	   const char *primary,
 	   const char *secondary,
-	   GdkPixbuf *pixbuf,
-	   gboolean show_action)
+	   const char *image_uri,
+	   gboolean playback)
 {
-	const char *icon_name;
 	GError *error = NULL;
+	NotifyNotification *notification;
 
-	if (notify_is_initted () == FALSE) {
-		GList *caps;
-
-		if (notify_init ("rhythmbox") == FALSE) {
-			g_warning ("libnotify initialization failed");
-			return;
-		}
-
-		/* ask the notification server if it supports actions */
-		caps = notify_get_server_caps ();
-		if (g_list_find_custom (caps, "actions", (GCompareFunc)g_strcmp0) != NULL) {
-			rb_debug ("notification server supports actions");
-			plugin->priv->notify_supports_actions = TRUE;
-		} else {
-			rb_debug ("notification server does not support actions");
-		}
-		rb_list_deep_free (caps);
+	if (init_notify (plugin) == FALSE) {
+		return;
 	}
 
 	update_status_icon_visibility (plugin, TRUE);
@@ -389,21 +442,29 @@ do_notify (RBStatusIconPlugin *plugin,
 	if (secondary == NULL)
 		secondary = "";
 
-	if (pixbuf == NULL)
-		icon_name = RB_APP_ICON;
-	else
-		icon_name = NULL;
+	if (playback) {
+		notification = plugin->priv->notification;
+	} else {
+		notification = NULL;
+	}
 
-	if (plugin->priv->notification == NULL) {
-		plugin->priv->notification = notify_notification_new (primary, secondary, icon_name, NULL);
+	if (notification == NULL) {
+#if LIBNOTIFY_VERSION_MINOR >= 7
+		notification = notify_notification_new (primary, secondary, RB_APP_ICON);
+#else
+		notification = notify_notification_new (primary, secondary, RB_APP_ICON, NULL);
+#endif
 
-		g_signal_connect_object (plugin->priv->notification,
+		g_signal_connect_object (notification,
 					 "closed",
 					 G_CALLBACK (notification_closed_cb),
 					 plugin, 0);
+		if (playback) {
+			plugin->priv->notification = notification;
+		}
 	} else {
 		notify_notification_clear_hints (plugin->priv->notification);
-		notify_notification_update (plugin->priv->notification, primary, secondary, icon_name);
+		notify_notification_update (plugin->priv->notification, primary, secondary, RB_APP_ICON);
 	}
 
 	switch (plugin->priv->icon_mode) {
@@ -414,31 +475,79 @@ do_notify (RBStatusIconPlugin *plugin,
 	case ICON_ALWAYS:
 	case ICON_OWNS_WINDOW:
 		rb_tray_icon_attach_notification (plugin->priv->tray_icon,
-						  plugin->priv->notification);
+						  notification);
 		break;
 
 	default:
 		g_assert_not_reached ();
 	}
 
-	notify_notification_set_timeout (plugin->priv->notification, timeout);
+	notify_notification_set_timeout (notification, timeout);
 
-	if (pixbuf != NULL) {
-		notify_notification_clear_hints (plugin->priv->notification);
-		notify_notification_set_icon_from_pixbuf (plugin->priv->notification, pixbuf);
+	if (image_uri != NULL) {
+		notify_notification_clear_hints (notification);
+		notify_notification_set_hint_string (notification,
+						     "image_path",
+						     image_uri);
 	}
 
-	notify_notification_clear_actions (plugin->priv->notification);
-	if (show_action && plugin->priv->notify_supports_actions) {
-		notify_notification_add_action (plugin->priv->notification,
-						"media-next",
+	notify_notification_clear_actions (notification);
+	if (playback && plugin->priv->notify_supports_actions) {
+		if (plugin->priv->notify_supports_icon_buttons) {
+			gboolean playing = FALSE;
+			rb_shell_player_get_playing (plugin->priv->shell_player, &playing, NULL);
+
+			notify_notification_add_action (notification,
+							"media-skip-backward",
+							_("Previous"),
+							(NotifyActionCallback) notification_previous_cb,
+							plugin,
+							NULL);
+			notify_notification_add_action (notification,
+							playing ? "media-playback-pause" : "media-playback-start",
+							playing ? _("Pause") : _("Play"),
+							(NotifyActionCallback) notification_playpause_cb,
+							plugin,
+							NULL);
+#if (LIBNOTIFY_VERSION_MINOR >= 7)
+			notify_notification_set_hint (notification,
+						      "action-icons",
+						      g_variant_new_boolean (TRUE));
+#else
+			notify_notification_set_hint_byte (notification,
+							   "action-icons",
+							   1);
+#endif
+		}
+
+		notify_notification_add_action (notification,
+						"media-skip-forward",
 						_("Next"),
 						(NotifyActionCallback) notification_next_cb,
 						plugin,
 						NULL);
 	}
 
-	if (notify_notification_show (plugin->priv->notification, &error) == FALSE) {
+	if (plugin->priv->notify_supports_persistence) {
+		const char *hint;
+
+		if (playback) {
+			hint = "resident";
+		} else {
+			hint = "transient";
+		}
+#if (LIBNOTIFY_VERSION_MINOR >= 7)
+		notify_notification_set_hint (notification,
+					      hint,
+					      g_variant_new_boolean (TRUE));
+#else
+		notify_notification_set_hint_byte (notification,
+						   hint,
+						   1);
+#endif
+	}
+
+	if (notify_notification_show (notification, &error) == FALSE) {
 		g_warning ("Failed to send notification (%s): %s", primary, error->message);
 		g_error_free (error);
 		update_status_icon_visibility (plugin, FALSE);
@@ -454,6 +563,10 @@ should_notify (RBStatusIconPlugin *plugin)
 {
 	gboolean visible;
 
+	if (plugin->priv->is_gnome_shell) {
+		return TRUE;
+	}
+
 	switch (plugin->priv->icon_mode) {
 	case ICON_NEVER:
 	case ICON_WITH_NOTIFY:
@@ -505,18 +618,23 @@ notify_playing_entry (RBStatusIconPlugin *plugin, gboolean requested)
 		   PLAYING_ENTRY_NOTIFY_TIME * 1000,
 		   plugin->priv->current_title,
 		   plugin->priv->current_album_and_artist,
-		   plugin->priv->notify_pixbuf,
+		   plugin->priv->notify_art_path,
 		   TRUE);
 }
 
 static void
-notify_custom (RBStatusIconPlugin *plugin, guint timeout, const char *primary, const char *secondary, GdkPixbuf *pixbuf, gboolean requested)
+notify_custom (RBStatusIconPlugin *plugin,
+	       guint timeout,
+	       const char *primary,
+	       const char *secondary,
+	       const char *image_uri,
+	       gboolean requested)
 {
 	if (requested == FALSE && should_notify (plugin) == FALSE) {
 		return;
 	}
 
-	do_notify (plugin, timeout, primary, secondary, pixbuf, FALSE);
+	do_notify (plugin, timeout, primary, secondary, image_uri, FALSE);
 }
 
 static void
@@ -541,7 +659,7 @@ notify_playing_entry (RBStatusIconPlugin *plugin, gboolean requested)
 }
 
 static void
-notify_custom (RBStatusIconPlugin *plugin, guint timeout, const char *primary, const char *secondary, GdkPixbuf *pixbuf, gboolean requested)
+notify_custom (RBStatusIconPlugin *plugin, guint timeout, const char *primary, const char *secondary, const char *image_uri, gboolean requested)
 {
 }
 
@@ -559,9 +677,15 @@ shell_notify_playing_cb (RBShell *shell, gboolean requested, RBStatusIconPlugin
 }
 
 static void
-shell_notify_custom_cb (RBShell *shell, guint timeout, const char *primary, const char *secondary, GdkPixbuf *pixbuf, gboolean requested, RBStatusIconPlugin *plugin)
+shell_notify_custom_cb (RBShell *shell,
+			guint timeout,
+			const char *primary,
+			const char *secondary,
+			const char *image_uri,
+			gboolean requested,
+			RBStatusIconPlugin *plugin)
 {
-	notify_custom (plugin, timeout, primary, secondary, pixbuf, requested);
+	notify_custom (plugin, timeout, primary, secondary, image_uri, requested);
 }
 
 /* tooltips */
@@ -694,17 +818,23 @@ update_current_playing_data (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 	char *album = NULL;
 	char *title = NULL;
 	GString *secondary;
+	gboolean playing = FALSE;
 
 	const char *artist_template = NULL;
 	const char *album_template = NULL;
 
 	g_free (plugin->priv->current_title);
 	g_free (plugin->priv->current_album_and_artist);
+	g_free (plugin->priv->notify_art_path);
 	plugin->priv->current_title = NULL;
 	plugin->priv->current_album_and_artist = NULL;
+	plugin->priv->notify_art_path = NULL;
 
-	if (entry == NULL)
+	if (entry == NULL) {
+		plugin->priv->current_title = g_strdup (_("Not Playing"));
+		plugin->priv->current_album_and_artist = g_strdup ("");
 		return;
+	}
 
 	secondary = g_string_sized_new (100);
 
@@ -775,11 +905,21 @@ update_current_playing_data (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 		g_free (escaped);
 	}
 
-	if (title != NULL)
-		plugin->priv->current_title = title;
-	else
+	if (title == NULL) {
 		/* Translators: unknown track title */
-		plugin->priv->current_title = g_strdup (_("Unknown"));
+		title = g_strdup (_("Unknown"));
+	}
+
+	rb_shell_player_get_playing (plugin->priv->shell_player, &playing, NULL);
+	if (playing) {
+		plugin->priv->current_title = title;
+	} else {
+		/* Translators: this is used in notification bubble text to indicate that
+		 * playback is currently paused.  %s is the song title.
+		 */
+		plugin->priv->current_title = g_strdup_printf (_("(Paused) %s"), title);
+		g_free (title);
+	}
 
 	plugin->priv->current_album_and_artist = g_string_free (secondary, FALSE);
 }
@@ -791,10 +931,6 @@ forget_pixbufs (RBStatusIconPlugin *plugin)
 		g_object_unref (plugin->priv->tooltip_pixbuf);
 		plugin->priv->tooltip_pixbuf = NULL;
 	}
-	if (plugin->priv->notify_pixbuf != NULL) {
-		g_object_unref (plugin->priv->notify_pixbuf);
-		plugin->priv->notify_pixbuf = NULL;
-	}
 }
 
 static void
@@ -826,6 +962,48 @@ is_playing_entry (RBStatusIconPlugin *plugin, RhythmDBEntry *entry)
 	return (entry == playing);
 }
 
+static void
+db_art_uri_metadata_cb (RhythmDB *db,
+			RhythmDBEntry *entry,
+			const char *field,
+			GValue *metadata,
+			RBStatusIconPlugin *plugin)
+{
+	guint time;
+
+	if (is_playing_entry (plugin, entry) == FALSE)
+		return;
+
+	if (G_VALUE_HOLDS (metadata, G_TYPE_STRING)) {
+		const char *uri = g_value_get_string (metadata);
+		if (g_str_has_prefix (uri, "file://")) {
+			char *path = g_filename_from_uri (uri, NULL, NULL);
+			if (g_strcmp0 (path, plugin->priv->notify_art_path) != 0) {
+				g_free (plugin->priv->notify_art_path);
+				plugin->priv->notify_art_path = path;
+			} else {
+				/* same art URI, ignore it */
+				g_free (path);
+				return;
+			}
+		} else {
+			/* unsupported art URI, ignore it */
+			return;
+		}
+	} else {
+		g_free (plugin->priv->notify_art_path);
+		plugin->priv->notify_art_path = NULL;
+	}
+
+	if (rb_shell_player_get_playing_time (plugin->priv->shell_player, &time, NULL)) {
+		if (time < PLAYING_ENTRY_NOTIFY_TIME) {
+			notify_playing_entry (plugin, FALSE);
+		}
+	} else {
+		notify_playing_entry (plugin, FALSE);
+	}
+}
+
 static void
 db_art_metadata_cb (RhythmDB *db,
 		    RhythmDBEntry *entry,
@@ -833,8 +1011,6 @@ db_art_metadata_cb (RhythmDB *db,
 		    GValue *metadata,
 		    RBStatusIconPlugin *plugin)
 {
-	guint time;
-
 	if (is_playing_entry (plugin, entry) == FALSE)
 		return;
 
@@ -851,21 +1027,10 @@ db_art_metadata_cb (RhythmDB *db,
 
 			scaled = rb_scale_pixbuf_to_size (pixbuf, GTK_ICON_SIZE_DIALOG);
 			plugin->priv->tooltip_pixbuf = create_tooltip_pixbuf (scaled);
-			plugin->priv->notify_pixbuf = scaled;
 		}
-
-		/* probably keep the full size thing for notifications?  hmm. */
 	}
 
 	rb_tray_icon_trigger_tooltip_query (plugin->priv->tray_icon);
-
-	if (rb_shell_player_get_playing_time (plugin->priv->shell_player, &time, NULL)) {
-		if (time < PLAYING_ENTRY_NOTIFY_TIME) {
-			notify_playing_entry (plugin, FALSE);
-		}
-	} else {
-		notify_playing_entry (plugin, FALSE);
-	}
 }
 
 static void
@@ -881,6 +1046,23 @@ db_stream_metadata_cb (RhythmDB *db,
 	update_current_playing_data (plugin, entry);
 }
 
+static void
+playing_changed_cb (RBShellPlayer *player,
+		    gboolean playing,
+		    RBStatusIconPlugin *plugin)
+{
+	RhythmDBEntry *entry;
+
+	entry = rb_shell_player_get_playing_entry (plugin->priv->shell_player);
+	if (entry != NULL) {
+		update_current_playing_data (plugin, entry);
+		rhythmdb_entry_unref (entry);
+	}
+
+	if (plugin->priv->notify_supports_persistence) {
+		notify_playing_entry (plugin, FALSE);
+	}
+}
 
 static void
 elapsed_changed_cb (RBShellPlayer *player,
@@ -938,7 +1120,7 @@ set_icon_geometry  (GdkWindow *window,
 
 	XChangeProperty (dpy,
 			 GDK_WINDOW_XID (window),
-			 gdk_x11_get_xatom_by_name_for_display (gdk_drawable_get_display (window),
+			 gdk_x11_get_xatom_by_name_for_display (gdk_window_get_display (window),
 								"_NET_WM_ICON_GEOMETRY"),
 			 XA_CARDINAL, 32, PropModeReplace,
 			 (guchar *)&data, 4);
@@ -1158,18 +1340,23 @@ config_notify_cb (GConfClient *client, guint connection_id, GConfEntry *entry, R
 {
 	if (g_str_equal (gconf_entry_get_key (entry), CONF_STATUS_ICON_MODE)) {
 
-		plugin->priv->icon_mode = gconf_value_get_int (gconf_entry_get_value (entry));
-		rb_debug ("icon mode changed to %d", plugin->priv->icon_mode);
+		plugin->priv->conf_icon_mode = gconf_value_get_int (gconf_entry_get_value (entry));
+		rb_debug ("icon mode changed to %d", plugin->priv->conf_icon_mode);
 
 		update_status_icon_visibility (plugin, FALSE);	/* maybe should remember if we're notifying.. */
 		sync_actions (plugin);
 
 		if (plugin->priv->icon_combo != NULL) {
 			plugin->priv->syncing_config_widgets = TRUE;
-			gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->icon_combo), plugin->priv->icon_mode);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->icon_combo), plugin->priv->conf_icon_mode);
 			plugin->priv->syncing_config_widgets = FALSE;
 		}
 
+		plugin->priv->icon_mode = plugin->priv->conf_icon_mode;
+		if (plugin->priv->is_gnome_shell) {
+			plugin->priv->icon_mode = ICON_NEVER;
+		}
+
 	} else if (g_str_equal (gconf_entry_get_key (entry), CONF_NOTIFICATION_MODE)) {
 		plugin->priv->notify_mode = gconf_value_get_int (gconf_entry_get_value (entry));
 		rb_debug ("notify mode changed to %d", plugin->priv->notify_mode);
@@ -1244,7 +1431,7 @@ impl_get_config_widget (RBPlugin *bplugin)
 				 G_CALLBACK (mouse_wheel_config_changed_cb),
 				 plugin, 0);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->notify_combo), plugin->priv->notify_mode);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->icon_combo), plugin->priv->icon_mode);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->icon_combo), plugin->priv->conf_icon_mode);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->wheel_combo), plugin->priv->wheel_mode);
 
 	g_object_unref (builder);
@@ -1260,6 +1447,9 @@ impl_activate (RBPlugin *bplugin,
 	RhythmDBEntry *entry;
 	GtkWindow *window;
 	char *uifile;
+#if GLIB_CHECK_VERSION(2, 26, 0)
+	GDBusConnection *bus;
+#endif
 
 	rb_debug ("activating status icon plugin");
 
@@ -1294,6 +1484,38 @@ impl_activate (RBPlugin *bplugin,
 		g_free (uifile);
 	}
 
+	/* are we running in GNOME Shell? */
+#if GLIB_CHECK_VERSION(2, 26, 0)
+
+	bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+	if (bus != NULL) {
+		GVariant *result;
+		result = g_dbus_connection_call_sync (bus,
+						      "org.freedesktop.DBus",
+						      "/org/freedesktop/DBus",
+						      "org.freedesktop.DBus",
+						      "GetNameOwner",
+						      g_variant_new ("(s)", "org.gnome.Shell"),
+						      G_VARIANT_TYPE ("(s)"),
+						      G_DBUS_CALL_FLAGS_NONE,
+						      -1,
+						      NULL,
+						      NULL);
+		if (result != NULL) {
+			rb_debug ("GNOME Shell is running");
+			plugin->priv->is_gnome_shell = TRUE;
+			g_variant_unref (result);
+		} else {
+			rb_debug ("GNOME Shell isn't running");
+			plugin->priv->is_gnome_shell = FALSE;
+		}
+		g_object_unref (bus);
+	}
+#else
+	rb_debug ("Unable to check if GNOME Shell is running");
+	plugin->priv->is_gnome_shell = FALSE;
+#endif
+
 	/* connect various things */
 	g_signal_connect_object (plugin->priv->shell, "visibility-changed", G_CALLBACK (visibility_changed_cb), plugin, 0);
 	g_signal_connect_object (plugin->priv->shell, "visibility-changing", G_CALLBACK (visibility_changing_cb), plugin, G_CONNECT_AFTER);
@@ -1301,10 +1523,13 @@ impl_activate (RBPlugin *bplugin,
 	g_signal_connect_object (plugin->priv->shell, "notify-custom", G_CALLBACK (shell_notify_custom_cb), plugin, 0);
 
 	g_signal_connect_object (plugin->priv->shell_player, "playing-song-changed", G_CALLBACK (playing_entry_changed_cb), plugin, 0);
+	g_signal_connect_object (plugin->priv->shell_player, "playing-changed", G_CALLBACK (playing_changed_cb), plugin, 0);
 	g_signal_connect_object (plugin->priv->shell_player, "elapsed-changed", G_CALLBACK (elapsed_changed_cb), plugin, 0);
 
 	g_signal_connect_object (plugin->priv->db, "entry_extra_metadata_notify::" RHYTHMDB_PROP_COVER_ART,
 				 G_CALLBACK (db_art_metadata_cb), plugin, 0);
+	g_signal_connect_object (plugin->priv->db, "entry_extra_metadata_notify::" RHYTHMDB_PROP_COVER_ART_URI,
+				 G_CALLBACK (db_art_uri_metadata_cb), plugin, 0);
 	g_signal_connect_object (plugin->priv->db, "entry_extra_metadata_notify::" RHYTHMDB_PROP_STREAM_SONG_TITLE,
 				 G_CALLBACK (db_stream_metadata_cb), plugin, 0);
 	g_signal_connect_object (plugin->priv->db, "entry_extra_metadata_notify::" RHYTHMDB_PROP_STREAM_SONG_ARTIST,
@@ -1323,7 +1548,11 @@ impl_activate (RBPlugin *bplugin,
 
 	maybe_upgrade_preferences (plugin);
 
-	plugin->priv->icon_mode = eel_gconf_get_integer (CONF_STATUS_ICON_MODE);
+	plugin->priv->conf_icon_mode = eel_gconf_get_integer (CONF_STATUS_ICON_MODE);
+	plugin->priv->icon_mode = plugin->priv->conf_icon_mode;
+	if (plugin->priv->is_gnome_shell) {
+		plugin->priv->icon_mode = ICON_NEVER;
+	}
 	plugin->priv->notify_mode = eel_gconf_get_integer (CONF_NOTIFICATION_MODE);
 	plugin->priv->wheel_mode = eel_gconf_get_integer (CONF_MOUSE_WHEEL_MODE);
 
@@ -1336,9 +1565,20 @@ impl_activate (RBPlugin *bplugin,
 	if (entry != NULL) {
 		update_current_playing_data (plugin, entry);
 		rhythmdb_entry_unref (entry);
+	} else {
+		/* display the (generic) app name rather than "Not Playing" for
+		 * the initial notification
+		 */
+		plugin->priv->current_title = g_strdup (_("Music Player"));
+		plugin->priv->current_album_and_artist = g_strdup ("");
 	}
 	update_tooltip (plugin);
 
+	/* for GNOME Shell, create a resident notification immediately */
+	if (plugin->priv->is_gnome_shell) {
+		notify_playing_entry (plugin, FALSE);
+	}
+
 	g_object_unref (ui_manager);
 	g_object_unref (window);
 }
@@ -1396,6 +1636,7 @@ impl_deactivate	(RBPlugin *bplugin,
 
 	if (plugin->priv->db != NULL) {
 		g_signal_handlers_disconnect_by_func (plugin->priv->db, db_art_metadata_cb, plugin);
+		g_signal_handlers_disconnect_by_func (plugin->priv->db, db_art_uri_metadata_cb, plugin);
 		g_signal_handlers_disconnect_by_func (plugin->priv->db, db_stream_metadata_cb, plugin);
 
 		g_object_unref (plugin->priv->db);
@@ -1424,9 +1665,11 @@ impl_deactivate	(RBPlugin *bplugin,
 	g_free (plugin->priv->current_title);
 	g_free (plugin->priv->current_album_and_artist);
 	g_free (plugin->priv->tooltip_markup);
+	g_free (plugin->priv->notify_art_path);
 	plugin->priv->current_title = NULL;
 	plugin->priv->current_album_and_artist = NULL;
 	plugin->priv->tooltip_markup = NULL;
+	plugin->priv->notify_art_path = NULL;
 
 	forget_pixbufs (plugin);
 }
