@@ -459,15 +459,24 @@ rb_podcast_manager_download_entry (RBPodcastManager *pd,
 	if ((status < RHYTHMDB_PODCAST_STATUS_COMPLETE) ||
 	    (status == RHYTHMDB_PODCAST_STATUS_WAITING)) {
 		RBPodcastManagerInfo *data;
-		if (status < RHYTHMDB_PODCAST_STATUS_COMPLETE) {
-			GValue status_val = { 0, };
-			g_value_init (&status_val, G_TYPE_ULONG);
-			g_value_set_ulong (&status_val, RHYTHMDB_PODCAST_STATUS_WAITING);
-			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
-			g_value_unset (&status_val);
+		GValue val = { 0, };
+		GTimeVal now;
 
-			rhythmdb_commit (pd->priv->db);
+		if (status < RHYTHMDB_PODCAST_STATUS_COMPLETE) {
+			g_value_init (&val, G_TYPE_ULONG);
+			g_value_set_ulong (&val, RHYTHMDB_PODCAST_STATUS_WAITING);
+			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &val);
+			g_value_unset (&val);
 		}
+
+		/* set last seen time so it shows up in the 'new downloads' subsource */
+		g_value_init (&val, G_TYPE_ULONG);
+		g_get_current_time (&now);
+		g_value_set_ulong (&val, now.tv_sec);
+		rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_LAST_SEEN, &val);
+		g_value_unset (&val);
+		rhythmdb_commit (pd->priv->db);
+
 		rb_debug ("Adding podcast episode %s to download list", get_remote_location (entry));
 
 		data = g_new0 (RBPodcastManagerInfo, 1);
@@ -497,6 +506,22 @@ rb_podcast_manager_entry_downloaded (RhythmDBEntry *entry)
 	return (status != RHYTHMDB_PODCAST_STATUS_ERROR && file_name != NULL);
 }
 
+gboolean
+rb_podcast_manager_entry_in_download_queue (RBPodcastManager *pd, RhythmDBEntry *entry)
+{
+	RBPodcastManagerInfo *info;
+	GList *l;
+
+	for (l = pd->priv->download_list; l != NULL; l = l->next) {
+		info = l->data;
+		if (info->entry == entry) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 void
 rb_podcast_manager_start_sync (RBPodcastManager *pd)
 {
@@ -1428,10 +1453,12 @@ download_progress (RBPodcastManagerInfo *data, guint64 downloaded, guint64 total
 			rhythmdb_entry_set (data->pd->priv->db, data->entry, RHYTHMDB_PROP_FILE_SIZE, &val);
 			g_value_unset (&val);
 
-			g_value_init (&val, G_TYPE_ULONG);
-			g_value_set_ulong (&val, RHYTHMDB_PODCAST_STATUS_COMPLETE);
-			rhythmdb_entry_set (data->pd->priv->db, data->entry, RHYTHMDB_PROP_STATUS, &val);
-			g_value_unset (&val);
+			if (total == 0 || downloaded >= total) {
+				g_value_init (&val, G_TYPE_ULONG);
+				g_value_set_ulong (&val, RHYTHMDB_PODCAST_STATUS_COMPLETE);
+				rhythmdb_entry_set (data->pd->priv->db, data->entry, RHYTHMDB_PROP_STATUS, &val);
+				g_value_unset (&val);
+			}
 
 			rb_podcast_manager_save_metadata (data->pd,
 							  data->entry);
