@@ -436,6 +436,7 @@ rhythmdb_property_model_set_property (GObject *object,
 			append_sort_property (model, RHYTHMDB_PROP_ALBUM);
 			break;
 		case RHYTHMDB_PROP_SUBTITLE:
+			append_sort_property (model, RHYTHMDB_PROP_ALBUM);
 			append_sort_property (model, RHYTHMDB_PROP_SUBTITLE);
 			break;
 		case RHYTHMDB_PROP_TITLE:
@@ -1380,6 +1381,7 @@ rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model,
 		n_elements = G_N_ELEMENTS (targets_artist);
 		break;
 	case RHYTHMDB_PROP_LOCATION:
+	case RHYTHMDB_PROP_SUBTITLE:		/* more or less */
 		targets = targets_location;
 		n_elements = G_N_ELEMENTS (targets_location);
 		break;
@@ -1431,9 +1433,9 @@ rhythmdb_property_model_column_get_type (void)
 
 	if (etype == 0)	{
 		static const GEnumValue values[] = {
-			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, "Property title"),
-			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, "Value priority"),
-			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER, "Track count"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, "property-title"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, "value-priority"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER, "track-count"),
 			{ 0, 0, 0 }
 		};
 
