@@ -53,8 +53,7 @@
 
 static void rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass);
 static void rb_missing_files_source_init (RBMissingFilesSource *source);
-static GObject *rb_missing_files_source_constructor (GType type, guint n_construct_properties,
-						     GObjectConstructParam *construct_properties);
+static void rb_missing_files_source_constructed (GObject *object);
 static void rb_missing_files_source_dispose (GObject *object);
 static void rb_missing_files_source_set_property (GObject *object,
 						  guint prop_id,
@@ -68,7 +67,7 @@ static void rb_missing_files_source_get_property (GObject *object,
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_song_properties (RBSource *source);
 static void impl_delete (RBSource *source);
-static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
+static void impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress);
 
 static void rb_missing_files_source_songs_show_popup_cb (RBEntryView *view,
 							 gboolean over_entry,
@@ -90,14 +89,17 @@ static void
 rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 
 	object_class->dispose = rb_missing_files_source_dispose;
-	object_class->constructor = rb_missing_files_source_constructor;
+	object_class->constructed = rb_missing_files_source_constructed;
 
 	object_class->set_property = rb_missing_files_source_set_property;
 	object_class->get_property = rb_missing_files_source_get_property;
 
+	page_class->get_status = impl_get_status;
+
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_false_function;
@@ -114,8 +116,6 @@ rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
 
-	source_class->impl_get_status = impl_get_status;
-
 	g_type_class_add_private (klass, sizeof (RBMissingFilesSourcePrivate));
 }
 
@@ -132,28 +132,24 @@ rb_missing_files_source_init (RBMissingFilesSource *source)
 					   "dialog-warning",
 					   size,
 					   0, NULL);
-	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
+	g_object_set (source, "pixbuf", pixbuf, NULL);
 	if (pixbuf != NULL) {
 		g_object_unref (pixbuf);
 	}
 }
 
-static GObject *
-rb_missing_files_source_constructor (GType type, guint n_construct_properties,
-				     GObjectConstructParam *construct_properties)
+static void
+rb_missing_files_source_constructed (GObject *object)
 {
 	GObject *shell_player;
 	RBMissingFilesSource *source;
-	RBMissingFilesSourceClass *klass;
 	RBShell *shell;
 	GPtrArray *query;
 	RhythmDBQueryModel *model;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
-	klass = RB_MISSING_FILES_SOURCE_CLASS (g_type_class_peek (RB_TYPE_MISSING_FILES_SOURCE));
-
-	source = RB_MISSING_FILES_SOURCE (G_OBJECT_CLASS (rb_missing_files_source_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_missing_files_source_parent_class, constructed, object);
+	source = RB_MISSING_FILES_SOURCE (object);
 
 	g_object_get (source,
 		      "shell", &shell,
@@ -172,7 +168,7 @@ rb_missing_files_source_constructor (GType type, guint n_construct_properties,
 				      	RHYTHMDB_PROP_HIDDEN,
 					TRUE,
 				      RHYTHMDB_QUERY_END);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	model = rhythmdb_query_model_new (source->priv->db, query,
 					  NULL, NULL, NULL, FALSE);
@@ -207,8 +203,6 @@ rb_missing_files_source_constructor (GType type, guint n_construct_properties,
 
 	g_object_set (source, "query-model", model, NULL);
 	g_object_unref (model);
-
-	return G_OBJECT (source);
 }
 
 static void
@@ -274,14 +268,14 @@ rb_missing_files_source_get_property (GObject *object,
  * it would be trivial to make it use any source type
  * that did file access checks for its contents.
  * 
- * Return value: the #RBMissingFiles source
+ * Return value: the #RBMissingFilesSource
  */
 RBSource *
 rb_missing_files_source_new (RBShell *shell,
 			     RBLibrarySource *library)
 {
 	RBSource *source;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	g_object_get (library, "entry-type", &entry_type, NULL);
 	source = RB_SOURCE (g_object_new (RB_TYPE_MISSING_FILES_SOURCE,
@@ -290,9 +284,8 @@ rb_missing_files_source_new (RBShell *shell,
 					  "shell", shell,
 					  "visibility", FALSE,
 					  "hidden-when-empty", TRUE,
-					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 	return source;
 }
 
@@ -302,7 +295,7 @@ rb_missing_files_source_songs_show_popup_cb (RBEntryView *view,
 					     RBMissingFilesSource *source)
 {
 	if (over_entry)
-		_rb_source_show_popup (RB_SOURCE (source), MISSING_FILES_SOURCE_SONGS_POPUP_PATH);
+		_rb_display_page_show_popup (RB_DISPLAY_PAGE (source), MISSING_FILES_SOURCE_SONGS_POPUP_PATH);
 }
 
 static void
@@ -344,12 +337,12 @@ rb_missing_files_source_songs_sort_order_changed_cb (RBEntryView *view,
 }
 
 static void
-impl_get_status (RBSource *asource, char **text, char **progress_text, float *progress)
+impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress)
 {
 	RhythmDBQueryModel *model;
 	gint count;
 
-	g_object_get (asource, "query-model", &model, NULL);
+	g_object_get (page, "query-model", &model, NULL);
 	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
 	g_object_unref (model);
 
