@@ -27,6 +27,21 @@
  *
  */
 
+/**
+ * SECTION:rb-static-playlist-source
+ * @short_description: Manually defined playlist class
+ *
+ * Static playlists are not defined by a query, but instead by manually selected
+ * and ordered tracks.
+ *
+ * This class is used for static playlists built from the user's library, and is
+ * also a base class for the play queue and for playlists on devices and network
+ * shares.
+ *
+ * It has some ability to track locations that are not yet present in the database
+ * and to add them to the playlist once they are added.
+ */
+
 #include "config.h"
 
 #include <string.h>
@@ -58,12 +73,12 @@ static void rb_static_playlist_source_get_property (GObject *object,
 
 /* source methods */
 static GList * impl_cut (RBSource *source);
-static void impl_paste (RBSource *asource, GList *entries);
+static RBTrackTransferBatch *impl_paste (RBSource *asource, GList *entries);
 static void impl_delete (RBSource *source);
 static void impl_search (RBSource *asource, RBSourceSearch *search, const char *cur_text, const char *new_text);
 static void impl_browser_toggled (RBSource *source, gboolean enabled);
 static void impl_reset_filters (RBSource *asource);
-static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
+static gboolean impl_receive_drag (RBDisplayPage *page, GtkSelectionData *data);
 static GList *impl_get_search_actions (RBSource *source);
 static guint impl_want_uri (RBSource *source, const char *uri);
 
@@ -145,6 +160,7 @@ static void
 rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
 
@@ -154,13 +170,14 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	object_class->set_property = rb_static_playlist_source_set_property;
 	object_class->get_property = rb_static_playlist_source_get_property;
 
+	page_class->receive_drag = impl_receive_drag;
+
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_cut = impl_cut;
 	source_class->impl_paste = impl_paste;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_search = impl_search;
 	source_class->impl_reset_filters = impl_reset_filters;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
@@ -192,13 +209,13 @@ rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 			g_object_add_weak_pointer (playlist_pixbuf,
 					   (gpointer *) &playlist_pixbuf);
 
-			rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+			g_object_set (source, "pixbuf", playlist_pixbuf, NULL);
 
 			/* drop the initial reference to the icon */
 			g_object_unref (playlist_pixbuf);
 		}
 	} else {
-		rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+		g_object_set (source, "pixbuf", playlist_pixbuf, NULL);
 	}
 }
 
@@ -263,7 +280,7 @@ rb_static_playlist_source_constructed (GObject *object)
 	RBPlaylistSource *psource;
 	RBEntryView *songs;
 	RBShell *shell;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_static_playlist_source_parent_class, constructed, object);
 
@@ -282,10 +299,10 @@ rb_static_playlist_source_constructed (GObject *object)
 	priv->paned = gtk_vpaned_new ();
 
 	g_object_get (source, "shell", &shell, NULL);
-	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
-							       "StaticPlaylistActions",
-							       NULL, 0,
-							       shell);
+	priv->action_group = _rb_display_page_register_action_group (RB_DISPLAY_PAGE (source),
+								     "StaticPlaylistActions",
+								     NULL, 0,
+								     shell);
 	if (gtk_action_group_get_action (priv->action_group,
 					 rb_static_playlist_source_radio_actions[0].name) == NULL) {
 		gtk_action_group_add_radio_actions (priv->action_group,
@@ -305,7 +322,10 @@ rb_static_playlist_source_constructed (GObject *object)
 	g_object_get (source, "entry-type", &entry_type, NULL);
 	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)),
 						entry_type);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	if (entry_type != NULL) {
+		g_object_unref (entry_type);
+	}
+
 	gtk_paned_pack1 (GTK_PANED (priv->paned), GTK_WIDGET (priv->browser), TRUE, FALSE);
 	g_signal_connect_object (priv->browser, "notify::output-model",
 				 G_CALLBACK (rb_static_playlist_source_browser_changed_cb),
@@ -336,8 +356,20 @@ rb_static_playlist_source_constructed (GObject *object)
 	gtk_widget_show_all (GTK_WIDGET (source));
 }
 
+/**
+ * rb_static_playlist_source_new:
+ * @shell: the #RBShell
+ * @name: the playlist name
+ * @sorting_name: the sorting name for the playlist (GConf key friendly)
+ * @local: if %TRUE, the playlist is local to the library
+ * @entry_type: type of database entries that can be added to the playlist.
+ *
+ * Creates a new static playlist source.
+ *
+ * Return value: new playlist.
+ */
 RBSource *
-rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sorting_name, gboolean local, RhythmDBEntryType entry_type)
+rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sorting_name, gboolean local, RhythmDBEntryType *entry_type)
 {
 	if (name == NULL)
 		name = "";
@@ -351,7 +383,6 @@ rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sor
 					"shell", shell,
 					"is-local", local,
 					"entry-type", entry_type,
-					"source-group", RB_SOURCE_GROUP_PLAYLISTS,
 					"search-type", RB_SOURCE_SEARCH_INCREMENTAL,
 					NULL));
 }
@@ -389,6 +420,13 @@ rb_static_playlist_source_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_static_playlist_source_load_from_xml:
+ * @source: an #RBStaticPlaylistSource
+ * @node: XML node to load from
+ *
+ * Loads the playlist contents from the specified XML document node.
+ */
 void
 rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNodePtr node)
 {
@@ -410,6 +448,15 @@ rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNode
 	}
 }
 
+/**
+ * rb_static_playlist_source_new_from_xml:
+ * @shell: the #RBShell
+ * @node: XML node containing playlist entries
+ *
+ * Constructs a new playlist from the given XML document node.
+ *
+ * Return value: playlist read from XML
+ */
 RBSource *
 rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 {
@@ -439,13 +486,15 @@ impl_cut (RBSource *asource)
 	return sel;
 }
 
-static void
+static RBTrackTransferBatch *
 impl_paste (RBSource *asource, GList *entries)
 {
 	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (asource);
 
 	for (; entries; entries = g_list_next (entries))
 		rb_static_playlist_source_add_entry (source, entries->data, -1);
+
+	return NULL;
 }
 
 static void
@@ -587,11 +636,11 @@ rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *browser,
 }
 
 static gboolean
-impl_receive_drag (RBSource *asource, GtkSelectionData *data)
+impl_receive_drag (RBDisplayPage *page, GtkSelectionData *data)
 {
 	GdkAtom type;
 	GList *list;
-	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (asource);
+	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (page);
 
 	type = gtk_selection_data_get_data_type (data);
 
@@ -745,6 +794,16 @@ _add_location_cb (GFile *file,
 	return TRUE;	
 }
 
+/**
+ * rb_static_playlist_source_add_location:
+ * @source: an #RBStaticPlaylistSource
+ * @location: location (URI) to add to the playlist
+ * @index: position at which to add the location (-1 to add at the end)
+ *
+ * If the location matches an entry in the database, the entry is added
+ * to the playlist.  Otherwise, if it identifies a directory, the contents
+ * of that directory are added.
+ */
 void
 rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 					const char *location,
@@ -767,6 +826,14 @@ rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 
 }
 
+/**
+ * rb_static_playlist_source_add_locations:
+ * @source: an #RBStaticPlaylistSource
+ * @locations: a #GList of strings to add
+ *
+ * Adds the locations specified in @locations to the playlist.
+ * See @rb_static_playlist_source_add_location for details.
+ */
 void
 rb_static_playlist_source_add_locations (RBStaticPlaylistSource *source,
 					 GList *locations)
@@ -779,6 +846,15 @@ rb_static_playlist_source_add_locations (RBStaticPlaylistSource *source,
 	}
 }
 
+/**
+ * rb_static_playlist_source_remove_location:
+ * @source: an #RBStaticPlaylistSource
+ * @location: location to remove
+ *
+ * Removes the specified location from the playlist.  This affects both
+ * the location map and the query model, whether an entry exists for the
+ * location or not.
+ */
 void
 rb_static_playlist_source_remove_location (RBStaticPlaylistSource *source,
 					   const char *location)
@@ -801,6 +877,14 @@ rb_static_playlist_source_remove_location (RBStaticPlaylistSource *source,
 	}
 }
 
+/**
+ * rb_static_playlist_source_add_entry:
+ * @source: an #RBStaticPlaylistSource
+ * @entry: entry to add to the playlist
+ * @index: position at which to add it (-1 to add at the end)
+ *
+ * Adds the specified entry to the playlist.
+ */
 void
 rb_static_playlist_source_add_entry (RBStaticPlaylistSource *source,
 				     RhythmDBEntry *entry,
@@ -812,6 +896,13 @@ rb_static_playlist_source_add_entry (RBStaticPlaylistSource *source,
 	rb_static_playlist_source_add_location_internal (source, location, index);
 }
 
+/**
+ * rb_static_playlist_source_remove_entry:
+ * @source: an #RBStaticPlaylistSource
+ * @entry: the entry to remove
+ *
+ * Removes the specified entry from the playlist.
+ */
 void
 rb_static_playlist_source_remove_entry (RBStaticPlaylistSource *source,
 					RhythmDBEntry *entry)
@@ -822,6 +913,14 @@ rb_static_playlist_source_remove_entry (RBStaticPlaylistSource *source,
 	rb_static_playlist_source_remove_location (source, location);
 }
 
+/**
+ * rb_static_playlist_source_move_entry:
+ * @source: an #RBStaticPlaylistSource
+ * @entry: the entry to move
+ * @index: new location for the entry
+ *
+ * Moves an entry within the playlist.
+ */
 void
 rb_static_playlist_source_move_entry (RBStaticPlaylistSource *source,
 				      RhythmDBEntry *entry,
@@ -909,4 +1008,3 @@ impl_want_uri (RBSource *source, const char *uri)
 
 	return 0;
 }
-
