@@ -7,7 +7,7 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -38,6 +38,7 @@
 
 #include "rhythmdb.h"
 #include "rhythmdb-tree.h"
+#include "rb-podcast-entry-types.h"
 
 /* test utils */
 gboolean waiting, signaled;
@@ -89,8 +90,8 @@ main (int argc, char **argv)
 	int i;
 
 	if (argc < 2) {
-		g_print ("using ~/.gnome2/rhythmbox/rhythmdb.xml\n");
-		name = g_strdup_printf ("%s/rhythmdb.xml", rb_dot_dir ());
+		name = g_build_filename (rb_user_data_dir(), "rhythmdb.xml", NULL);
+		g_print ("using %s\n", name);
 	} else {
 		name = g_strdup (argv[1]);
 	}
@@ -101,10 +102,9 @@ main (int argc, char **argv)
 	rb_threads_init ();
 	gtk_set_locale ();
 	gtk_init (&argc, &argv);
-	gnome_vfs_init ();
 	rb_debug_init (FALSE);
 	rb_refstring_system_init ();
-	rb_file_helpers_init ();
+	rb_file_helpers_init (TRUE);
 
 	GDK_THREADS_ENTER ();
 
@@ -136,7 +136,6 @@ main (int argc, char **argv)
 	
 	rb_file_helpers_shutdown ();
         rb_refstring_system_shutdown ();
-        gnome_vfs_shutdown ();
 
 
 	rb_profile_end ("load test");
