@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* rbcellrendererpixbuf.c
- *
- * arch-tag: Implementation of Rhythmbox pixbuf GtkTreeView cell renderer
  *
  * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
  * Copyright (C) 2002  Jorn Baayen <jorn@nl.linux.org>
@@ -28,6 +26,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include "gseal-gtk-compat.h"
 #include "rb-cell-renderer-pixbuf.h"
 #include "rb-cut-and-paste-code.h"
 
@@ -89,9 +88,10 @@ static guint rb_cell_renderer_pixbuf_signals [LAST_SIGNAL] = { 0 };
 static void
 rb_cell_renderer_pixbuf_init (RBCellRendererPixbuf *cellpixbuf)
 {
-
 	/* set the renderer able to be activated */
-	GTK_CELL_RENDERER (cellpixbuf)->mode = GTK_CELL_RENDERER_MODE_ACTIVATABLE;
+	g_object_set (cellpixbuf,
+		      "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE,
+		      NULL);
 }
 
 static void
@@ -212,6 +212,8 @@ rb_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
   gint pixbuf_height = 0;
   gint calc_width;
   gint calc_height;
+  gint xpad, ypad;
+  gfloat xalign, yalign;
 
   if (cellpixbuf->pixbuf)
     {
@@ -219,23 +221,26 @@ rb_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
       pixbuf_height = gdk_pixbuf_get_height (cellpixbuf->pixbuf);
     }
 
-  calc_width = (gint) GTK_CELL_RENDERER (cellpixbuf)->xpad * 2 + pixbuf_width;
-  calc_height = (gint) GTK_CELL_RENDERER (cellpixbuf)->ypad * 2 + pixbuf_height;
+  gtk_cell_renderer_get_padding (GTK_CELL_RENDERER (cellpixbuf), &xpad, &ypad);
+  calc_width = xpad * 2 + pixbuf_width;
+  calc_height = ypad * 2 + pixbuf_height;
 
   if (x_offset) *x_offset = 0;
   if (y_offset) *y_offset = 0;
 
   if (cell_area && pixbuf_width > 0 && pixbuf_height > 0)
     {
+      gtk_cell_renderer_get_alignment (GTK_CELL_RENDERER (cellpixbuf), &xalign, &yalign);
+
       if (x_offset)
 	{
-	  *x_offset = GTK_CELL_RENDERER (cellpixbuf)->xalign * (cell_area->width - calc_width - (2 * GTK_CELL_RENDERER (cellpixbuf)->xpad));
-	  *x_offset = MAX (*x_offset, 0) + GTK_CELL_RENDERER (cellpixbuf)->xpad;
+	  *x_offset = xalign * (cell_area->width - calc_width - (2 * xpad));
+	  *x_offset = MAX (*x_offset, 0) + xpad;
 	}
       if (y_offset)
 	{
-	  *y_offset = GTK_CELL_RENDERER (cellpixbuf)->yalign * (cell_area->height - calc_height - (2 * GTK_CELL_RENDERER (cellpixbuf)->ypad));
-	  *y_offset = MAX (*y_offset, 0) + GTK_CELL_RENDERER (cellpixbuf)->ypad;
+	  *y_offset = yalign * (cell_area->height - calc_height - (2 * ypad));
+	  *y_offset = MAX (*y_offset, 0) + ypad;
 	}
     }
 
@@ -260,17 +265,18 @@ rb_cell_renderer_pixbuf_render (GtkCellRenderer    *cell,
   GdkRectangle pix_rect;
   GdkRectangle draw_rect;
   GtkStateType state;
+  gint xpad, ypad;
 
   if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
     {
-      if (GTK_WIDGET_HAS_FOCUS (widget))
+      if (gtk_widget_has_focus (widget))
         state = GTK_STATE_SELECTED;
       else
         state = GTK_STATE_ACTIVE;
     }
   else
     {
-      if (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE)
+      if (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE)
         state = GTK_STATE_INSENSITIVE;
       else
         state = GTK_STATE_NORMAL;
@@ -287,22 +293,17 @@ rb_cell_renderer_pixbuf_render (GtkCellRenderer    *cell,
 
   pix_rect.x += cell_area->x;
   pix_rect.y += cell_area->y;
-  pix_rect.width -= cell->xpad * 2;
-  pix_rect.height -= cell->ypad * 2;
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+  pix_rect.width -= xpad * 2;
+  pix_rect.height -= ypad * 2;
 
-  if (gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect))
-    gdk_draw_pixbuf (window,
-		     NULL,
-		     cellpixbuf->pixbuf,
-		     /* pixbuf 0, 0 is at pix_rect.x, pix_rect.y */
-		     draw_rect.x - pix_rect.x,
-		     draw_rect.y - pix_rect.y,
-		     draw_rect.x,
-		     draw_rect.y,
-		     draw_rect.width,
-		     draw_rect.height,
-		     GDK_RGB_DITHER_NORMAL,
-		     0, 0);
+  if (gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect)) {
+    cairo_t *cr = gdk_cairo_create (window);
+    gdk_cairo_set_source_pixbuf (cr, cellpixbuf->pixbuf, pix_rect.x, pix_rect.y);
+    gdk_cairo_rectangle (cr, &draw_rect);
+    cairo_paint (cr);
+    cairo_destroy (cr);
+  }
 }
 
 static gboolean
@@ -314,21 +315,32 @@ rb_cell_renderer_pixbuf_activate (GtkCellRenderer *cell,
 				  GdkRectangle *cell_area,
 				  GtkCellRendererState flags)
 {
-	int mouse_x, mouse_y, icon_width;
-	RBCellRendererPixbuf *cellpixbuf = (RBCellRendererPixbuf *) cell;
+  int mouse_x, mouse_y;
+  RBCellRendererPixbuf *cellpixbuf = (RBCellRendererPixbuf *) cell;
 
-	g_return_val_if_fail (RB_IS_CELL_RENDERER_PIXBUF (cellpixbuf), FALSE);
+  g_return_val_if_fail (RB_IS_CELL_RENDERER_PIXBUF (cellpixbuf), FALSE);
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &icon_width, NULL);
-	gtk_widget_get_pointer (widget, &mouse_x, &mouse_y);
-	gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (widget),
-							   mouse_x, mouse_y,
-							   &mouse_x, &mouse_y);
+  if (event == NULL) {
+    return FALSE;
+  }
+  /* only handle mouse events */
+  switch (event->type) {
+    case GDK_BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      break;
+    default:
+      return FALSE;
+  }
 
-	/* ensure the user clicks within the good cell */
-	if (mouse_x - cell_area->x >= 0
-	    && mouse_x - cell_area->x <= cell_area->width) {
-		g_signal_emit (G_OBJECT (cellpixbuf), rb_cell_renderer_pixbuf_signals [PIXBUF_CLICKED], 0, path);
-	}
-	return TRUE;
+  gtk_widget_get_pointer (widget, &mouse_x, &mouse_y);
+  gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (widget),
+						     mouse_x, mouse_y,
+						     &mouse_x, &mouse_y);
+
+  /* ensure the user clicks within the good cell */
+  if (mouse_x - cell_area->x >= 0
+      && mouse_x - cell_area->x <= cell_area->width) {
+    g_signal_emit (G_OBJECT (cellpixbuf), rb_cell_renderer_pixbuf_signals [PIXBUF_CLICKED], 0, path);
+  }
+  return TRUE;
 }
