@@ -309,7 +309,7 @@ rb_rating_realize (GtkWidget *widget)
 	GdkWindow *window;
 	int attributes_mask;
 
-	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+	gtk_widget_set_realized (widget, TRUE);
 
 	gtk_widget_get_allocation (widget, &allocation);
 
@@ -372,7 +372,12 @@ rb_rating_expose (GtkWidget *widget,
 	rating = RB_RATING (widget);
 
 	window = gtk_widget_get_window (widget);
-	gdk_drawable_get_size (window, &width, &height);
+#if GTK_CHECK_VERSION(2, 23, 0)
+	width = gdk_window_get_width (window);
+	height = gdk_window_get_height (window);
+#else
+	gdk_drawable_get_size (GDK_DRAWABLE (window), &width, &height);
+#endif
 
 	gtk_widget_style_get (widget, "focus-line-width", &focus_width, NULL);
 	if (gtk_widget_has_focus (widget)) {
